
extern "C" {
#include "psim.h"
}

#include "operators.h"
#include "eta.h"
//#include <primex.h>



#define MESON_ETA "eta"
#define MESON_MASS_ETA 0.54730                     // in GeV
#define MESON_ETA_PRIME "eta prime"
#define MESON_MASS_ETA_PRIME 0.95777               // in GeV
#define MESON_MASS_RHO 0.7699                      // in GeV
#define MESON_MASS_OMEGA OMEGA_MASS                // in GeV
#define TARGET_A 12.0
#define TARGET_Z 6.0

extern float MESON_MASS;
extern float TARGET_MASS;
extern char *MESON;
extern cross_table_ptr_t *eta_cross;



extern "C" {

int eta_prime_He4(float E);
int eta_prime_C12(float E);
float calc_q(float theta, float m, float M, float k);

void gsvert_(float *VERT,int *zero,int *zero,float *UBUF,int *zero,int *NVTX);
void gskine_(float *PLAB,int *IPART,int *NVTX,float *UBUF,int *zero,int *NT);
}

// Routine declarations
int eta_prime(float E);
int GenerateEta(float E,int flags,float *theta,float *phi,float *momentum);
float RANDOM(void);
Vect4* DecayEtaPrime(float theta,float phi,float p,int *N,int **types,int *decay_mech);
Vect4* DecayEta(float theta,float phi,float p,int *N,int **types,int *decay_mech);
Vect4* DecayRho(float theta,float phi,float p,int *N,int **types,int *decay_mech);
Vect4* DecayOmega(float theta,float phi,float p,int *N,int **types,int *decay_mech);
Vect3 Rotate(Vect3 p,float theta,float phi);
Vect4 Boost(Vect4 v,Vect3 b);
void find_theta_phi(Vect3 v, float *theta,float *phi_ret);
void find_theta_phi(Vect4 v, float *theta,float *phi_ret);



int eta_prime_He4(float E)
{
   eta_cross=eta_prime_cross_He4;
   MESON_MASS=MESON_MASS_ETA_PRIME;
   MESON=MESON_ETA_PRIME;
   return eta_prime(E);
}

int eta_prime_C12(float E)
{
   eta_cross=eta_prime_cross_C12;
   MESON_MASS=MESON_MASS_ETA_PRIME;
   MESON=MESON_ETA_PRIME;
   return eta_prime(E);
}


int eta_prime(float E)
{
	int i;
   int mech,decay_mech;
   int *types;
   float theta,phi,p;
   float theta1,phi1,theta2,phi2;
   float p1,p2;
   Vect4 *decay_parts;
   int N;
   float VERT[3]={0.0,0.0,0.0};
   float PLAB[3]={0.0,0.0,0.0};
   float UBUF;
   float beam_size;
   int IPART;
   int NVTX,NT,HID;
   float thetas[10];
   float Etotal=0.0;

   // 4He= 3.728 (this is from (4.002603amu)(931.502 MeV/amu) )
   // 12C=11.178 (this is from (12amu)(931.502 MeV/amu) )
	TARGET_MASS=11.178;

   // Generate an eta prime
	mech=GenerateEta(E,PRIM_CROSS|NUCL_CROSS|INTER_CROSS,&theta,&phi,&p);
   initial_4vect.E=sqrt(pow(p,2.0)+pow(MESON_MASS,2.0));
   initial_4vect.x=p*sin(theta)*cos(phi);
   initial_4vect.y=p*sin(theta)*sin(phi);
   initial_4vect.z=p*cos(theta);
   
   // The bremstrahlung generator can generate events below the
   // eta production threshold. If this happens, the eta momentum
   // will be NaN.
   if(isnan(p))return -1;
   if(E<MESON_MASS)
      printf("E<MESON_MASS!!! p=%f E=%f theta=%f phi=%f MESON_MASS=%f\n"
         ,p,E,theta,phi,MESON_MASS);

   // Decay the eta prime
   decay_parts=DecayEtaPrime(theta,phi,p,&N,&types,&decay_mech);
   if(N<2 || N>5){cerr<<"Error in "<<__FILE__<<" at line "<<__LINE__<<"\n";return -2;}

   event_subtype=mech;
   event_decay_mech=decay_mech;

   /* determine initial position of eta and */
   /* randomly place in a 250 micron radius */
   /* at the center of target               */
   beam_size=RANDOM()*250.0E-4;
   phi=RANDOM()*2.0*PI;
   VERT[0] = beam_size*cos(phi);
   VERT[1] = beam_size*sin(phi);
   VERT[2] = thargt_.thrpos[2] + ((RANDOM()*50.0)-20.0);
   gsvert_(VERT,&zero,&zero,&UBUF,&zero,&NVTX);
   initial_position.x=VERT[0];
   initial_position.y=VERT[1];
   initial_position.z=VERT[2];
   
   // Define the resulting decay products to GEANT
   for(i=0;i<N;i++){
      IPART=types[i];
      find_theta_phi(decay_parts[i], &theta, &phi);
      thetas[i]=theta;
      theta*=DEG2RAD;
      phi  *=DEG2RAD;
      p=decay_parts[i].p();
      PLAB[0]=p*cos(phi)*sin(theta);
      PLAB[1]=p*sin(phi)*sin(theta);
      PLAB[2]=p*cos(theta);
		Etotal+=decay_parts[i].E;
		
      if(!flags[generate_only])gskine_(PLAB,&IPART,&NVTX,&UBUF,&zero,&NT);
   }
   if(decay_mech==0)hf2(5003,thetas[0],thetas[1],1.0);

//printf("Etotal=%f  E=%f   frac=%2.2f%%\n\n",Etotal,E,Etotal*100.0/E);

   return 0;
}


// =============================================================
// Decay the meson. Right now, there is only one decay channel
// supported. The meson->2photon channel
// =============================================================
Vect4* DecayEtaPrime(float theta,float phi,float p,int *Num,int **types,int *mech)
{
	int i,j,N,N_direct;
	static Vect4 parts[32],tp;
   static int parttype[32];
	Vect3 beta,beta_decay;
	float E,b;
	float ph,th;
   float chan_weight[32];
   int Num_chans=0;
   float r,m2;
   float PCM[3][4]={{0.0,0.0,0.0,0.0},{0.0,0.0,0.0,0.0},{0.0,0.0,0.0,0.0}};
	int *decay_types,decay_mech,Ndecay;
	Vect4 *decay_parts;
   Vect4 Pparent=Vect4(0.0, 0.0, 0.0, 0.0);
   Vect4 Pdecay=Vect4(0.0, 0.0, 0.0, 0.0);
	
   
	// Create a 3 vector for the beta of the meson in the lab frame
	E=sqrt(pow(MESON_MASS,2.0)+pow(p,2.0));
	b=p/E;
	beta.px=b*(sin(theta)*cos(phi));
	beta.py=b*(sin(theta)*sin(phi));
	beta.pz=b*(cos(theta));

   // Deterimine decay channel
   chan_weight[Num_chans++]=0.4370; // pi+, pi-, eta
   chan_weight[Num_chans++]=0.3020; // rho, gamma
   chan_weight[Num_chans++]=0.2080; // pi0, pi0, eta
   chan_weight[Num_chans++]=0.0302; // omega, gamma
   chan_weight[Num_chans++]=0.0212; // gamma, gamma
   for(i=1;i<Num_chans;i++)chan_weight[i]+=chan_weight[i-1];
   r=chan_weight[Num_chans-1]*RANDOM();
   for(i=Num_chans-1;i>=0;i--)
      if(chan_weight[i]>r)*mech=i;

   // Create decay products in rest frame of decaying meson
   N=0;
   switch(*mech){
      case 0: // pi+, pi-, eta
         gdeca3(MESON_MASS ,PI_CHARGED_MASS ,PI_CHARGED_MASS ,ETA_MASS ,(float**)PCM);
         parttype[N++]=ptype_pion_plus;
         parttype[N++]=ptype_pion_minus;
         parttype[N++]=ptype_eta;
         break;
      case 1: // rho, gamma
         gdeca2(MESON_MASS ,RHO_MASS ,0.0 ,(float**)PCM);
         parttype[N++]=ptype_rho;
         parttype[N++]=ptype_gamma;
         break;
      case 2: // pi0, pi0, eta
         gdeca3(MESON_MASS ,PI_ZERO_MASS ,PI_ZERO_MASS ,ETA_MASS ,(float**)PCM);
         parttype[N++]=ptype_pion_zero;
         parttype[N++]=ptype_pion_zero;
         parttype[N++]=ptype_eta;
         break;
      case 3: // omega, gamma
         gdeca2(MESON_MASS ,OMEGA_MASS ,0.0 ,(float**)PCM);
         parttype[N++]=ptype_omega;
         parttype[N++]=ptype_gamma;
         break;
      case 4: // gamma, gamma
         gdeca2(MESON_MASS ,0.0 ,0.0 ,(float**)PCM);
         parttype[N++]=ptype_gamma;
         parttype[N++]=ptype_gamma;
         break;
      default:
         cerr<<"\nError in "<<__FILE__<<" near line "<<__LINE__<<"\n\n";
         cerr<<"*mech="<<*mech<<"  r="<<r<<"\n";
   }

	// Boost the decay products into the lab frame
	N_direct=N;
	for(i=0;i<N;i++){

      if(i<N_direct){
      	parts[i].px=PCM[i][0];
      	parts[i].py=PCM[i][1];
      	parts[i].pz=PCM[i][2];
      	parts[i].E =PCM[i][3];
		}
      
   	// Some of the decay products above must be decayed further
      Ndecay=0;
      find_theta_phi(parts[i], &theta, &phi);
      theta*=DEG2RAD;
      phi*=DEG2RAD;
      p=parts[i].p();
      switch(parttype[i]){
			case ptype_eta:
				MESON_MASS=MESON_MASS_ETA;
				decay_parts=DecayEta(theta,phi,p,&Ndecay,&decay_types,&decay_mech);
				break;
			case ptype_rho:
				MESON_MASS=MESON_MASS_RHO;
				decay_parts=DecayRho(theta,phi,p,&Ndecay,&decay_types,&decay_mech);
				break;
			case ptype_omega:
				MESON_MASS=MESON_MASS_OMEGA;
				decay_parts=DecayOmega(theta,phi,p,&Ndecay,&decay_types,&decay_mech);
				break;
		}
		MESON_MASS=MESON_MASS_ETA_PRIME;
		if(Ndecay){


			parts[i]=decay_parts[0];
			parttype[i]=decay_types[0];
			for(j=1;j<Ndecay;j++){
				 /* the rest will be boosted at the end of the "i" loop */
				parts[N]=decay_parts[j];
				Pdecay=Pdecay+Boost(parts[N],beta);
				parttype[N++]=decay_types[j];
			}
			
			Pdecay=Pdecay+Boost(decay_parts[0],beta);
		}

		parts[i]=Boost(parts[i],beta);
		m2=parts[i].Mass2();
		if(fabs(m2) <1.0E-3 && fabs(m2)!=0.0)parts[i].E=parts[i].p()+0.000000001;

		Pparent=Pparent+parts[i];

		//printf("m%d = %f\n",i,parts[i].Mass());

	}
   *types=parttype;
	*Num=N;

	//if(Pdecay.Mass()!=0.0)printf("Mass=%f  Mass_decay=%f\n",Pparent.Mass(),Pdecay.Mass());

	return parts;
}





