


#include "psim.h"

typedef int generator_proc_ptr(float E);

int num_generators=0;
unsigned long generator_range_start[256];
unsigned long generator_range_stop[256];
char generator_names[32][32];
generator_proc_ptr *procs[256];
float initial_photon_energy;
vect4 initial_4vect,initial_position;
int BEAM_ENERGY=11.0;

/* function declarations */
extern float brehmstrahlung(float E);
extern generator_proc_ptr photon;
extern generator_proc_ptr electron;
extern generator_proc_ptr pairs;
extern generator_proc_ptr pi0;
extern generator_proc_ptr eta_He4;
extern generator_proc_ptr eta_C12;
extern generator_proc_ptr eta_prime_C12;
extern generator_proc_ptr eta_prime_He4;
extern generator_proc_ptr rho;
extern generator_proc_ptr omega;

float        photon_weight=1.0;
float      electron_weight=0.0;
float          pair_weight=0.0;
float           pi0_weight=0.0;
float       eta_He4_weight=0.0;
float       eta_C12_weight=0.0;
float eta_prime_C12_weight=0.0;
float eta_prime_He4_weight=0.0;
float           rho_weight=0.0;
float         omega_weight=0.0;

void init_generators(void)
{
   int i;
   unsigned long range;
   double probs[256],tot_probs=0.0;
   char names[32][32];

	num_generators=0;
	
   /*------- make generators list ---------*/
   /* beam photon */
   procs[num_generators] = photon;
   probs[num_generators] = photon_weight;
   strcpy(names[num_generators++],"photon");
   /* beam electron */
   procs[num_generators] = electron;
   probs[num_generators] = electron_weight;
   strcpy(names[num_generators++],"electron");
   /* pair production */
   procs[num_generators] = pairs;
   probs[num_generators] = pair_weight;
   strcpy(names[num_generators++],"pairs");
   /* Primakoff pi0 production */
   procs[num_generators] = pi0;
   probs[num_generators] = pi0_weight;
   strcpy(names[num_generators++],"pi0");
   /* Primakoff eta production on He4*/
   procs[num_generators] = eta_He4;
   probs[num_generators] = eta_He4_weight;
   strcpy(names[num_generators++],"eta on He4");
   /* Primakoff eta production on C12*/
   procs[num_generators] = eta_C12;
   probs[num_generators] = eta_C12_weight;
   strcpy(names[num_generators++],"eta on C12");
   /* Primakoff eta prime production on C12*/
   procs[num_generators] = eta_prime_C12;
   probs[num_generators] = eta_prime_C12_weight;
   strcpy(names[num_generators++],"eta prime on C12");
   /* Primakoff eta prime production on He4*/
   procs[num_generators] = eta_prime_He4;
   probs[num_generators] = eta_prime_He4_weight;
   strcpy(names[num_generators++],"eta prime on He4");
   /* Phasespace rho production on C12*/
   procs[num_generators] = rho;
   probs[num_generators] = rho_weight;
   strcpy(names[num_generators++],"rho on C12");
   /* Phasespace omega production on C12*/
   procs[num_generators] = omega;
   probs[num_generators] = omega_weight;
   strcpy(names[num_generators++],"omega on C12");


   /*------- determine random number ranges --------*/
   for(i=0;i<num_generators;i++)tot_probs+=probs[i];

   printf("\n-------------- Activating Generators ---------------\n");
   generator_range_start[0]=0;
   for(i=0;i<num_generators;i++){
      range=(unsigned long)((double)(RAND_MAX)*(probs[i]/tot_probs));
      
      generator_range_start[i+1]=generator_range_start[i]+range;
      generator_range_stop[i]=generator_range_start[i+1];
      strcpy(generator_names[i],names[i]);

      if(range==0)continue;
      
      printf("Generators: %d: %s\n\tstart:%u\n\tstop:%u\n\trange:%u\n"
         ,i,names[i]
         ,generator_range_start[i],generator_range_stop[i],range);
   }
   
}

void generate_event(void)
{
   unsigned long s;
   int i,HID;
   float f;
   static int running=0;

   if(flags[verbose] && !running)printf("  Generating event...");fflush(stdout);
   running=1;
   
   s=RAND_MAX&random();
   
   for(i=0;i<num_generators;i++){
      if(s>=generator_range_start[i] && s<generator_range_stop[i])
         break;
   }
   
   if(i>=num_generators){
      printf("\n\n\tThis error message indicates a bug in the");
      printf(" generate_event routine.(shame on you Dave!)\n");
      printf(" s=%ld  generator_range_start[0]=%ld  generator_range_stop[0]=%ld\n"
         ,s,generator_range_start[0],generator_range_stop[0]);
      /*exit(-1);*/
      generate_event();
      return;
   }
   
   initial_photon_energy=brehmstrahlung(BEAM_ENERGY);
   
   /* if generated photon is below some limit, try again */
   if(initial_photon_energy<1.0E0){
      generate_event();
      return;
   }
   
   HID=1000;
   event_type=i;
   f=(float)event_type;
   hf1_(&HID,&f,&fone);
   
   /* Call the specific event generator */
   event_subtype=event_decay_mech=0;
   if(procs[i](initial_photon_energy))generate_event();
   
   hf1_(&one,&initial_photon_energy,&fone);
   
   /* Make "E" component of position the radius from center */
   initial_position.E = sqrt(
       (initial_position.x * initial_position.x)
      +(initial_position.y * initial_position.y)
      +(initial_position.z * initial_position.z));

   if(flags[verbose] && running)printf("done\n");
   running=0;
}

/* generate random number between two limits */
float ranl(float lower,float upper)
{
   float r;
   
   r=(float)((RAND_MAX-1)&random())/(float)RAND_MAX;
   r*=upper-lower;
   r+=lower;
   
   return r;
}

void SelectGenerator(int who)
{

	photon_weight
	=electron_weight
	=pair_weight
	=pi0_weight
	=eta_He4_weight
	=eta_C12_weight
	=eta_prime_C12_weight
	=eta_prime_He4_weight
	=rho_weight
	=omega_weight=0.0;

	if(who==0)photon_weight       =1.0;
	if(who==1)electron_weight     =1.0;
	if(who==2)pair_weight         =1.0;
	if(who==3)pi0_weight          =1.0;
	if(who==4)eta_He4_weight      =1.0;
	if(who==5)eta_C12_weight      =1.0;
	if(who==6)eta_prime_C12_weight=1.0;
	if(who==7)eta_prime_He4_weight=1.0;
	if(who==8)rho_weight          =1.0;
	if(who==9)omega_weight        =1.0;

	
}







