
extern "C" {
#include "psim.h"
}

#include "operators.h"



#define MESON_RHO "rho"
#define MESON_MASS_RHO 0.7699                      // in GeV
#define TARGET_A 12.0
#define TARGET_Z 6.0

extern float MESON_MASS;
extern float TARGET_MASS;
extern char *MESON;

extern "C" {

int rho(float E);
float calc_q(float theta, float m, float M, float k);

void gsvert_(float *VERT,int *zero,int *zero,float *UBUF,int *zero,int *NVTX);
void gskine_(float *PLAB,int *IPART,int *NVTX,float *UBUF,int *zero,int *NT);
}

// Routine declarations
int GenerateRho(float E,int flags,float *theta,float *phi,float *momentum);
float RANDOM(void);
Vect4* DecayRho(float theta,float phi,float p,int *N,int **types,int *decay_mech);
Vect3 Rotate(Vect3 p,float theta,float phi);
Vect4 Boost(Vect4 v,Vect3 b);
void find_theta_phi(Vect3 v, float *theta,float *phi_ret);
void find_theta_phi(Vect4 v, float *theta,float *phi_ret);




int rho(float E)
{
	int i;
   int mech,decay_mech;
   int *types;
   float theta,phi,p;
   float theta1,phi1,theta2,phi2;
   float p1,p2;
   Vect4 *decay_parts;
   int N;
   float VERT[3]={0.0,0.0,0.0};
   float PLAB[3]={0.0,0.0,0.0};
   float UBUF;
   float beam_size;
   int IPART;
   int NVTX,NT,HID;
   float thetas[10];

   MESON_MASS=MESON_MASS_RHO;
   MESON=MESON_RHO;
	E=BEAM_ENERGY;

   // 4He= 3.728 (this is from (4.002603amu)(931.502 MeV/amu) )
   // 12C=11.178 (this is from (12amu)(931.502 MeV/amu) )
	TARGET_MASS=11.178;

	mech=GenerateRho(E,0,&theta,&phi,&p);
   initial_4vect.E=sqrt(pow(p,2.0)+pow(MESON_MASS,2.0));
   initial_4vect.x=p*sin(theta)*cos(phi);
   initial_4vect.y=p*sin(theta)*sin(phi);
   initial_4vect.z=p*cos(theta);

   // The bremstrahlung generator can generate events below the
   // eta production threshold. If this happens, the eta momentum
   // will be NaN.
   if(!finite(p))return -1;
   if(!finite(theta))return -1;
   if(!finite(phi))return -1;
   if(E<MESON_MASS)
      printf("E<MESON_MASS!!! p=%f E=%f theta=%f phi=%f MESON_MASS=%f\n"
         ,p,E,theta,phi,MESON_MASS);

   // Decay the rho
   decay_parts=DecayRho(theta,phi,p,&N,&types,&decay_mech);
   if(N!=2 && N!=3){cerr<<"Error in"<<__FILE__<<" at line "<<__LINE__<<"\n";return -2;}
   if(decay_parts[0].Mass()!=0.0
      || decay_parts[1].Mass()!=0.0){/*return -5;*/}

   event_subtype=mech;
   event_decay_mech=decay_mech;

   /* determine initial position of rho and */
   /* randomly place in a 250 micron radius */
   /* at the center of target               */
   beam_size=RANDOM()*250.0E-4;
   phi=RANDOM()*2.0*PI;
   VERT[0] = beam_size*cos(phi);
   VERT[1] = beam_size*sin(phi);
   VERT[2] = thargt_.thrpos[2] + ((RANDOM()*50.0)-20.0);
   gsvert_(VERT,&zero,&zero,&UBUF,&zero,&NVTX);
   initial_position.x=VERT[0];
   initial_position.y=VERT[1];
   initial_position.z=VERT[2];
   
   // Define the resulting decay products to GEANT
   for(i=0;i<N;i++){
      IPART=types[i];
      find_theta_phi(decay_parts[i], &theta, &phi);
      thetas[i]=theta;
      theta*=DEG2RAD;
      phi  *=DEG2RAD;
      p=decay_parts[i].p();
      PLAB[0]=p*cos(phi)*sin(theta);
      PLAB[1]=p*sin(phi)*sin(theta);
      PLAB[2]=p*cos(theta);

      if(!flags[generate_only])gskine_(PLAB,&IPART,&NVTX,&UBUF,&zero,&NT);
   }

   return 0;
}


int GenerateRho(float E,int flags,float *theta,float *phi,float *momentum)
{
	int i,j;
	int index,mech=0;
	float B,C,a,b,c,dM2;
	float p_rho,p_A,Ei;
	float theta_rho,theta_A,phi_rho;
	Vect4 initial_state,rho4v;
	Vect3 cm_beta;

   // Don't have rho production cross-section as function of theta
   // right now. Just generate it isotropically in CM frame with e'
   // going at zero degrees

   // First, find initial energy in CM frame
   initial_state.E=E+TARGET_MASS;
   initial_state.px=0.0;
   initial_state.py=0.0;
   initial_state.pz=-E;

	cm_beta.px=cm_beta.py=0.0;
	cm_beta.pz=-initial_state.pz/initial_state.E;
	initial_state=Boost(initial_state,cm_beta);
	Ei=initial_state.E;

	// Randomly pick directions for rho and nucleus in CM frame
	theta_rho=RANDOM()*PI;
	theta_A  =RANDOM()*PI;
	phi_rho  =RANDOM()*PI*2.0;

	// Calculate magnitude of rho momentum in CM frame
	B = cos(theta_rho)*(1.0-(sin(theta_rho)/sin(theta_A)));
	C = (B*B) - pow(sin(theta_rho)/sin(theta_A),2.0) + 1.0;
	dM2 = pow(MESON_MASS,2.0)-pow(TARGET_MASS,2.0);
	a = pow(dM2,2.0);
	b = 2.0*C*(dM2-(2.0*B*B*MESON_MASS*MESON_MASS/C));
	c = (C*C) - (4.0*B*B);
	p_rho = (-b + sqrt((b*b) - (4.0*a*c)) )/2.0*a;
	p_rho = 1.0/sqrt(p_rho);

	// Calculate rest of kinematics from rho's

	// Boost rho back into lab frame
	rho4v.px = p_rho*sin(theta_rho)*cos(phi_rho);
	rho4v.py = p_rho*sin(theta_rho)*sin(phi_rho);
	rho4v.pz = p_rho*cos(theta_rho);
	rho4v.E  = sqrt(pow(MESON_MASS,2.0)+pow(p_rho,2.0));
	rho4v=Boost(rho4v,cm_beta);
	find_theta_phi(rho4v, theta, phi);
	*momentum = rho4v.p();
	
	
	return mech;
}

// =============================================================
// Decay the meson. Right now, there is only one decay channel
// supported. The rho->2pion channel
// =============================================================
Vect4* DecayRho(float theta,float phi,float p,int *Num,int **types,int *mech)
{
	int i,j,N;
	static Vect4 parts[32],tp;
   static int parttype[32];
	Vect3 beta;
	float E,b;
	float ph,th;
   float chan_weight[32];
   int Num_chans=0;
   float r;
   float PCM[3][4]={{0.0,0.0,0.0,0.0},{0.0,0.0,0.0,0.0},{0.0,0.0,0.0,0.0}};
   
	// Create a 3 vector for the beta of the meson in the lab frame
	E=sqrt(pow(MESON_MASS,2.0)+pow(p,2.0));
	b=p/E;
	beta.px=b*(sin(theta)*cos(phi));
	beta.py=b*(sin(theta)*sin(phi));
	beta.pz=b*(cos(theta));

   // Deterimine decay channel
   chan_weight[Num_chans++]=1.0000; // pi+, pi-
   for(i=1;i<Num_chans;i++)chan_weight[i]+=chan_weight[i-1];
   r=chan_weight[Num_chans-1]*RANDOM();
   for(i=Num_chans-1;i>=0;i--)
      if(chan_weight[i]>r)*mech=i;

   // Create decay products in rest frame of decaying meson
   N=0;
   switch(*mech){
      case 0: // pi+,pi-
         gdeca2(MESON_MASS ,PI_CHARGED_MASS ,PI_CHARGED_MASS ,(float**)PCM);
         parttype[N++]=ptype_pion_plus;
         parttype[N++]=ptype_pion_minus;
         break;
      default:
         cerr<<"\nError in "<<__FILE__<<" near line "<<__LINE__<<"\n\n";
         cerr<<"*mech="<<*mech<<"  r="<<r<<"\n";
   }


	// Boost the decay products into the lab frame
	for(i=0;i<N;i++){
      parts[i].px=PCM[i][0];
      parts[i].py=PCM[i][1];
      parts[i].pz=PCM[i][2];
      parts[i].E =PCM[i][3];
      
		parts[i]=Boost(parts[i],beta);
		//parts[i]=Boost(parts[i],Vect3(0.0, 0.0, 0.0));
		if(fabs(1.0 - (parts[i].E/parts[i].p())) <1.0E-5)parts[i].E=parts[i].p();
	}

   *mech=1;
   *types=parttype;
	*Num=N;
	return parts;
}






