


#include "psim.h"

/* set detector trheshold at 1 MeV */
#define E_thresh 1.0E-3


int Nclus,clust[2048];

cluster_t clusters[max_clusters];
int Nsaved_clusters;


void FindCluster(int x,int y,int ICOP_scale,int ICOP_offset);
int  RecordCluster(int HID);
void XYposition(int detector,float *X,float *Y);
void MergeClusters(void);


void FindClusters(void)
{
   int HID, i, j;

   /*---------- cluster sizes etc.. ----------*/
   Nsaved_clusters=0;
   
   /* find clusters in outer */
   HID=1202;
   if(sets[1]->HITS>0){
      for(i=1;i<=25;i++){
         for(j=1;j<=25;j++){
            Nclus=0;
            FindCluster(i,j,25,0);
            RecordCluster(HID);
         }
      }
   }
   /* find clusters in inner */
   HID=1203;
   if(sets[2]->HITS>0){
      for(i=1;i<=22;i++){
         for(j=1;j<=22;j++){
            Nclus=0;
            FindCluster(i,j,22,1000);
            RecordCluster(HID);
         }
      }
   }
   
   /* this will combine clusters that are found in the outer */
   /* and inner detectors */
   MergeClusters();

}


/* this routine will search all detector elements around element  */
/* x,y (including the element at x,y) and look for neighbors with */
/* hits. If it finds a neighbor who was hit, it calls itself so   */
/* all the neighbor's neighbors will also be searched for hits.   */
void FindCluster(int x,int y,int ICOP_scale,int ICOP_offset)
{
   int i,j,k;
   int X,Y;
   int INDEX;
   float *SHWDET  = cheren_.shwdet;
   
   for(i=-1;i<=+1;i++){
      for(j=-1;j<=+1;j++){
         X=x+i;
         Y=y+j;
         if(X<1 )continue;
         if(X>ICOP_scale)continue;
         if(Y<1 )continue;
         if(Y>ICOP_scale)continue;
         
         INDEX = X + ((Y-1)*ICOP_scale) + ICOP_offset;
         if(SHWDET[INDEX] > 0.0){
            for(k=0;k<Nclus;k++){
               if(clust[k]==INDEX)break;
            }
            
            /* if above loop was broken out of, then */
            /* this hit has already been counted     */
            if(k<Nclus)continue;
            
            /* add this hit to current cluster */
            clust[Nclus++]=INDEX;
            
            /* Call this routine again centered around this hit */
            FindCluster(X,Y,ICOP_scale,ICOP_offset);
         }
      }
   }
}



/* checks if a cluster that was found is unique for this event and */
/* stores the info and returns one if so. Returns zero if cluster  */
/* is not unique or has zero elements.                             */
int RecordCluster(int HID)
{
   int i,j,k;
   cluster_t *cl;
   float A,X,Y;
   float *SHWDET  = cheren_.shwdet;

   if(Nclus<=0)return 0;

   /* check if cluster was already found */
   for(i=0;i<Nsaved_clusters;i++){
      cl=&clusters[i];
      for(j=0;j<Nclus;j++)if(clust[j]==cl->who[0])return 0;
   }
   
   /* this must be a new cluster */
   cl=&clusters[Nsaved_clusters++];
   cl->N=Nclus;
   for(j=0;j<Nclus;j++)cl->who[j]=clust[j];
   A=(float)Nclus;
   hf1_(&HID,&A,&fone);   

   /* find center of cluster */
   cl->X=0.0;
   cl->Y=0.0;
   cl->E=0.0;

   for(k=0;k<cl->N;k++){
      XYposition(cl->who[k],&X,&Y);
      cl->E+=SHWDET[cl->who[k]];
      cl->X+=X*cl->E;
      cl->Y+=Y*cl->E;
   }
   cl->X/=cl->E;
   cl->Y/=cl->E;

   return 1;
}


/* this simple routine returns the x and y positions of the center */
/* of the given detector number. See GEOMETRY/calorimeter.f        */
void XYposition(int detector,float *X,float *Y)
{
   int i,j;
   
   /* outer */
   if(detector<=625){
      i=1 + ((detector-1)%25);
      j=1 + ((detector-1)/25);
      *X=((float)i*4.0 ) - 52.0;
      *Y=((float)j*4.0 ) - 52.0;
   }else if(detector<=1484){
      detector-=1000;
      i=1 + ((detector-1)%22);
      j=1 + ((detector-1)/22);
      *X=((float)i*2.0 ) - 23.0;
      *Y=((float)i*2.0 ) - 23.0;
   }
}


/* this routine looks for clusters in the insertion that can be   */
/* connected with clusters in the outer detectors and merges them */
/* into a single cluster.                                         */
void MergeClusters(void)
{
   float X,Y;
   int i,j,k,m;
   cluster_t *cl1,*cl2;
   
   
   /* for now, just look for clusters whose centers are <= 6 cm apart */
   for(m=0;m<Nsaved_clusters-1;m++){   
      for(k=m+1;k<Nsaved_clusters;k++){
         cl1=&clusters[m];
         cl2=&clusters[k];
         X=cl1->X - cl2->X;
         Y=cl1->Y - cl2->Y;
         if(fabs(X*X +Y*Y)>6.0)continue;
         
         /* merge clusters */
         cl1->X = ((cl1->X*cl1->E) + (cl2->X*cl2->E))/(cl1->E+cl2->E);
         cl1->Y = ((cl1->Y*cl1->E) + (cl2->Y*cl2->E))/(cl1->E+cl2->E);
         cl1->E = cl1->E + cl2->E;
         for(i=0;i<cl2->N;i++)cl1->who[cl1->N++]=cl2->who[i];
         
         /* remove cluster */
         for(i=k;i<Nsaved_clusters-1;i++)clusters[i]=clusters[i+1];
         Nsaved_clusters--;
         
         /* re-call this function to be safe */
         MergeClusters();
         return;
      }
   }

}


















