
/**************************************************************/
/* write_binary_event.c
/*
/* April 7,1999
/*
/* Write the raw event to a compressed gzip file.
/*
/* This routine is only called if the "-gz" option is passed.
/* It is called from HISTOGRAM/fill-histos.c .The compression
/* is done "on-the-fly" by using routines in the zlib library.
/*
/* The output can be re-played using "psim_book" (yet to be
/* written). This should save on certain simulation tasks
/* since the showers are tracked only once and the results
/* saved to the file.
/*
/**************************************************************/



#include "psim.h"


void write_binary_header(void);


void* write_binary_event(void)
{
   static int initialized=0;
   static raw_event_t re;
   float *SHWDET  = cheren_.shwdet;
   float *TSHWDET = cheren_.tshwdet;
   int i;
   float Edep=0.0;
   
   /* make sure file is open for writing */
   if(flags[output_gzip] && !gzfile){
      printf("\n No gzip file opened!\n\n");
      exit(-1);
   }
   
   /* Write header in file on first event */
   if(!initialized){
      if(flags[output_gzip])write_binary_header();
      initialized=1;
   }
   
   /* zero entire structure so un-used parts are easily compressed */
   bzero((char*)&re,sizeof(raw_event_t));

   
   /* fill data structure with event values */
   strcpy(re.header,"NEW EVENT:\n");
   strcpy(re.trailer,"END EVENT:\n");
   re.event=IEVENT;
   
   memcpy(re.SHWDET,SHWDET,sizeof(float)*2048);
   memcpy(re.TSHWDET,TSHWDET,sizeof(float)*2048);
   
   re.type=event_type;
   re.subtype=event_subtype;
   re.decay_mech=event_decay_mech;
   re.Ephoton=initial_photon_energy;
   for(i=0;i<2000;i++)Edep+=SHWDET[i];
   re.Edeposited=Edep;
   re.initial_4vect=initial_4vect;
   re.initial_position=initial_position;
   
   re.Npart_infos=Npart_infos<max_part_infos ? Npart_infos:max_part_infos;
   memcpy(re.part_info, part_info, sizeof(part_info_t)*max_part_infos);


   return &re;
}



/**************************************************/
/* Write an ASCII header in the file to give some */
/* basic info about the conditions of how this    */
/* file was generated.                            */
/**************************************************/

void write_binary_header(void)
{
   char *ptr,str[8192];
   time_t t;
   int i;
   long range;
   FILE *file;
   
   sprintf(str,"PSIM -simulated data for PrimEx experiment\n\n");
   gzwrite(gzfile,str,strlen(str));

   t=time(NULL);
   sprintf(str,"generated on: %s",ctime(&t));
   gzwrite(gzfile,str,strlen(str));
   
   ptr=getenv("HOST");
   sprintf(str,"HOST:%s\n",ptr==NULL ? "unknown":ptr);
   gzwrite(gzfile,str,strlen(str));
   
   ptr=getenv("USER");
   sprintf(str,"USER:%s\n",ptr==NULL ? "unknown":ptr);
   gzwrite(gzfile,str,strlen(str));
   
   sprintf(str,"generators:%d\n",num_generators);
   gzwrite(gzfile,str,strlen(str));
   sprintf(str,"\tweights:\n");
   gzwrite(gzfile,str,strlen(str));
   for(i=0;i<num_generators;i++){
      range=generator_range_stop[i]-generator_range_start[i];
      sprintf(str,"\t%d)%f  %s\n"
         ,i
         ,(double)range/(double)RAND_MAX
         ,generator_names[i]);
      gzwrite(gzfile,str,strlen(str));
   }
   
   sprintf(str,"Geometry type: ");
	if(GEOMETRY_TYPE==0)strcat(str,"pion");
	else if(GEOMETRY_TYPE==1)strcat(str,"eta_electron");
	else if(GEOMETRY_TYPE==2)strcat(str,"eta_photon");
	else sprintf(str,"%sunknown %d",GEOMETRY_TYPE);
	strcat(str,"\n");
   gzwrite(gzfile,str,strlen(str));
   
   sprintf(str,"event size in bytes:%d\n",sizeof(raw_event_t));
   gzwrite(gzfile,str,strlen(str));

   sprintf(str,"\n==============pr_back.dat file follows:==============\n");
   gzwrite(gzfile,str,strlen(str));
   file=fopen("pr_back.dat","r");
   if(!file){
      sprintf(str,"<unavailable>\n");
   }else{
      fread(str,8190,1,file);
      fclose(file);
   }
   gzwrite(gzfile,str,strlen(str));

   sprintf(str,"\n\nEvents follow:\n");
   gzwrite(gzfile,str,strlen(str));

}







