

#include "../common.h"


/* This is where the ntuples and histograms are booked */


/* declare a global handle to an array of type "subsets"   */
/* and a number of elements (memory will be allocated below */
subset *sets[max_sets];
int nsets=0;


subset* AddSubset(char *name);


void uhinit_(void)
{
   char fname[32];
   int i,j;
   subset *s;
   long ISEED1,ISEED2,ISEQ;
   
   printf("********************************************\n");
   printf("****  SEEDING RANDOM NUMBER GENERATORS  ****\n");
   printf("********************************************\n");
   printf("\n");
   srandom(time(NULL));
   ISEED1=time(NULL);
   ISEED2=getpid();
   while(ISEQ<1 || ISEQ>=215)ISEQ=random();
   grndmq(ISEED1,ISEED2,ISEQ," ");
   printf("ISEED1=%ld  ISEED2=%ld  ISEQ=%ld\n\n",ISEED1,ISEED2,ISEQ);
   
   printf("********************************************\n");
   printf("******   BOOKING HISTOS AND NTUPLES  *******\n");
   printf("********************************************\n");
   printf("\n");
   if(strlen(OUTFILE)==0)
      sprintf(OUTFILE,"r_b_int_%3.3d.hist",gcflag_.idrun);
   printf(" saving to file: %s\n",OUTFILE);
   fflush(stdout);
   
   /* open output file */
   if(hropen(1,"HBOOK",OUTFILE,"N",1024,0))return;


   /*******************************************/
   /*             define histograms           */
   /*******************************************/
   
   hbook1(1,"Initial photon energy (GeV)",100,0.0,6.0,0.0);

   hbook1(2,"Total Energy deposited in Detector (MeV)",100,0.0,6000.0,0.0);
   
   hbook1(3,"Volume copy number",2048,0.0,2048.0,0.0);
   
   /* Pair Counters */
   hbook1(2001,"Upstream, left pair counter",2048,0.0,0.02048,0.0);
   
   hcopy(2001,2002,"Downstream, left pair counter");
   
   hcopy(2001,2003,"Upstream, right pair counter");
   
   hcopy(2001,2004,"Downstream, right pair counter");

   /*******************************************/
   /*             define subsets              */
   /*******************************************/
   
   /* make ntuple nt1 (CWN) */
   hbnt(nt1,"E_and_T"," ");
   
   /* add some variables */
   hbname(nt1,"FLAG",&(flags[pair_prod]),"pair_prod:I");
   hbname(nt1,"FLAG",&(flags[pair_prod_in_tgt]),"pair_prod_in_t:I");
   
   /* all detectors */
   s=AddSubset("ALL");
   if(s){
      for(i=1;i<=25;i++)
         for(j=1;j<=25;j++)
            if(i<7 || i>18 || j<7 || j>18)s->list[s->Num++]=(25*(j-1))+i;

      for(i=1;i<=22;i++)
         for(j=1;j<=22;j++)
            if(i<9 || i>12 || j<9 || j>12)s->list[s->Num++]=(22*(j-1))+i+1000;         
   }
   
   /* outer array */
   s=AddSubset("OUTER");
   if(s){
      for(i=1;i<=25;i++)
         for(j=1;j<=25;j++)
            if(i<7 || i>18 || j<7 || j>18)s->list[s->Num++]=(25*(j-1))+i; 
   }

   /* insertion array */
   s=AddSubset("INNER");
   if(s){
      for(i=1;i<=22;i++)
         for(j=1;j<=22;j++)
            if(i<9 || i>12 || j<9 || j>12)s->list[s->Num++]=(22*(j-1))+i+1000;
   }

   /* Pair counters */
   s=AddSubset("PC");
   if(s){
      s->list[s->Num++]=2001;
      s->list[s->Num++]=2002;
      s->list[s->Num++]=2003;
      s->list[s->Num++]=2004;
   }
   /* Beam Left Pair counters */
   s=AddSubset("PCL");
   if(s){
      s->list[s->Num++]=2001;
      s->list[s->Num++]=2002;
   }
   /* Beam Right Pair counters */
   s=AddSubset("PCR");
   if(s){
      s->list[s->Num++]=2003;
      s->list[s->Num++]=2004;
   }
   
   
}


subset* AddSubset(char *name){
   subset *s;
   char chform[512],chblok[32],lcname[32],hits_str[32];
   int i;
   
   printf("Adding subset \"%s\" to ntuple %d\n\n",name,nt1);
   /* allocate memory for subset data structure */
   s=(subset*)malloc(sizeof(subset));
   if(s==NULL)return NULL;
   
   /* add subset to list of sets */
   sets[nsets++]=s;
   s->Num=0;
   strcpy(s->name,name);
   
   /* prepare some strings */
   strcpy(chblok,name);
   
   /* use lower case of set name as prefix for variable names */
   strcpy(lcname,name);
   for(i=0;i<strlen(lcname);i++)lcname[i]=(char)tolower((int)lcname[i]);
   
   /* name of variable representing number of hits/elements in arrays */
   sprintf(hits_str,"%s_hits",lcname);
   
   /* string defining variables to be added to ntuple */
   sprintf(chform,"%s[0,1023]:I,%s_E:R,%s_T:R,%s_Eelem(%s):R,%s_Telem(%s):R"
      ,hits_str,lcname,lcname,lcname,hits_str,lcname,hits_str);
   
   /* add the subsets for this variable to the ntuple */
   hbname(nt1,chblok,&(s->HITS),chform);
   
   /* return a pointer to the newly allocated subset structure */
   /* so the "Num" and "list" fields can be filled */
   return s;
}



