

#include "psim.h"


/* Global Variables */
int pawc_[NWPAWC];
int gcbank_[NWGEAN];
gcflax_t gcflax_;
FILE *gzfile=NULL;
int GEOMETRY_TYPE=0;
int flags[NUMFLAGS];
int debug=0;
int track_flags[NUM_TRACKFLAGS];
double run_time = 0.0;
float EVENT_WRITE_TRHESHOLD=0.0;
long N_EVENT=0L,N_GOOD_EVENT=0L;
int nt1=100,nt2=10,nt3=11,nt4=12,nt5=13;
int event_type,event_subtype,event_decay_mech;
magcut_t magcut_={0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,0.01,1};
posito_t posito_={150.0 ,0.0 ,0.0 ,0.0 ,0.0};
zplane_t zplane_={};
brmst1_t brmst1_;
phbmen_t phbmen_={1.0E-4,1.0};
cheren_t cheren_;
nascom_t nascom_;

float vtheta=45.0,vphi=45.0,vpsi=0.0,vu=18.0,vv=18.0,vsu=0.025,vsv=0.025;
int HIDE=0,SHADE=0;

int zero=0,one=1,two=2,three=3,four=4,five=5,six=6;
int seven=7,eight=8,nine=9,ten=10;
float fzero=0.,fone=1.,ftwo=2.,fthree=3.,ffour=4.,ffive=5.;
float fsix=6.,fseven=7.,feight=8.,fnine=9.,ften=10.;

/* function declarations */
void uglast(void);
void uginit(void);
void PrintUsage(void);
void EHandler(int sig);
void ParseCommandLine(int narg,char *argv[]);
void OpenOutputFile(void);

void GUI_init(void);

/* This is to satisfy the Linux g77 compiler */
int MAIN__(int narg,char *argv[]){main(narg,argv);}

int main(int narg,char *argv[])
{
   int i,N,K;
   int c;
   
   /*Try and exit gracefully should any errors occur*/
   signal(SIGPIPE,EHandler);
   signal(SIGTERM,EHandler);
   signal(SIGINT,EHandler);
   signal(SIGFPE,EHandler);

   /*------- parse command line args --------*/
   for(i=0;i<NUMFLAGS;i++)flags[i]=0;
   ParseCommandLine(narg,argv);
   
   /*-------------- Initialize --------------*/
   N=NWGEAN;
   gzebra_(&N);
   N=-NWPAWC;
   if(flags[global_section])
      hlimap_(&N,"PRIM",strlen("PRIM"));
   else
      hlimit_(&N);

   uginit();
   
   init_generators();
   
   if(flags[fill_histograms])book_histos();   

   OpenOutputFile();
   
   if(flags[use_gui]){      
      GUI_init();
      flags[skip_simulation]=1;
      flags[pause_at_end]=0;
   }

   /*-------------- Simulation loop ---------------*/
   if(!flags[skip_simulation]){
      printf("\n----- Begining simulation -----\n\n");
      grun_();
      printf("\n----- Simulation done -----\n\n");
   }
   /*----------- Cleanup and exit -----------*/

   if(flags[pause_at_end]){
      printf("\n\n\t\tHit <enter> to exit\n");
      c=getchar();
   }
   
   uglast();

   
   return 0;
}

void uglast(void)
{
   close_histos();
   glast_();
   igend_();
}


void PrintUsage(void)
{
   printf("\nUsage:\n");
   printf("\tsimulation [-v -g -p -dt -gs -ss -h] [-sv view]\n\n");
   printf("\t\t-b    batch mode output (no counter)\n");
   printf("\t\t-v    verbose output\n");
   printf("\t\t-gui  invoke Graphical User Interface\n");
   printf("\t\t-g    turn on graphics\n");
   printf("\t\t-p    pause at end (useful to keep graphics from\n");
   printf("\t\t      disappearing too early)\n");
   printf("\t\t-dt   draw tracks (*REALLY* slows things down)\n");
   printf("\t\t-nct  don't draw tracks (*REALLY* slows things down)\n");
   printf("\t\t-fh   fill histograms during simulation\n");
   printf("\t\t-gs   map to global memory section. (by default, the\n");
   printf("\t\t      histograms are not mapped to the global section \"PRIM\")\n");
   printf("\t\t-ss   skip simulation (useful for just drawing)\n");
   printf("\t\t-go   generate events only (skip tracking)\n");
   printf("\t\t-sac  stop tracking particles when they hit the calorimeter\n");
   printf("\t\t-gz   write events to file compressed with gzip\n");
   printf("\t\t-E#   Energy needed in calorimeter (in GeV) to write event\n");
   printf("\t\t-sv   set view parameters. This requires an argument which\n");
   printf("\t\t      is a string of the form \"theta phi psi uv vv su sv\".\n");
   printf("\t\t      (where theta,phi,psi,uv, and vv are numbers describing\n");
   printf("\t\t      the view). Graphics is implied by this option.\n");
   printf("\t\t      e.g.\n\n");
   printf("\t\t      psim -sv \"45.0 45.0 0.0 18.0 18.00.032 0.032\"\n\n");
   printf("\t\t      or\n\n");
   printf("\t\t      psim -ss -p -sv \"170.0 130.0 0.0 5.0 3.5 0.095 0.095\"\n\n");
   printf("\t\t-T#   Set time in hours for program to run.\n");
   printf("\t\t-G#   Set calorimeter geometry (-G0=pi0 -G1=eta)\n");
   printf("\t\t-gen# Select generator. The following generators are defined:\n");
	printf("\t\t           0 bremstrahlung photon\n");
	printf("\t\t           1 electron at full beam energy\n");
	printf("\t\t           2 pair production using bremstrahlung energies\n");
	printf("\t\t           3 Primakoff pi0\n");
	printf("\t\t           4 Eta production on He4 (Primakoff,Nucl. coh., Int.)\n");
	printf("\t\t           5 Eta production on C12 (Primakoff,Nucl. coh., Int.)\n");
	printf("\t\t           6 Eta' production on C12 (Primakoff,Nucl. coh., Int.)\n");
	printf("\t\t           7 Eta' production on He4 (Primakoff,Nucl. coh., Int.)\n");
	printf("\t\t           8 Rho electroproduction (0 degree electron)\n");
	printf("\t\t           9 Omega electroproduction (0 degree electron)\n");
   printf("\t\t-h    prints this usage statement\n");
   
   printf("\n");
   printf("The output of the simulation is written to a file \n");
   printf("with a unique name based on the name of the execution\n");
   printf("host (e.g. \"sim_lepton001.hbook\") in the current directory.\n");
   printf("By default, a hbook file is created for output. If the \"-gz\"\n");
   printf("option is passed, compressed binary output will be written to\n");
   printf("a file with a \".gz\" suffix for later analysis.\n");
   printf("The input configuration file is called \"pr_back.dat\"\n");
   printf("and should reside in the current directory.\n");
   printf("Histograms can be viewed while the simulation is running\n");
   printf("by entering the following command inside PAW:\n");
   printf("\nPAW > global_sec PRIM\n\n");
   
   printf("\n");
   exit(-1);
}

void EHandler(int sig)
{
   flags[quit]=1;
   printf("\nReport: signal received of type ");
   if(sig==SIGPIPE)printf("SIGPIPE\n");
   if(sig==SIGTERM)printf("SIGTERM\n");
   if(sig==SIGINT)printf("SIGINT\n");
   if(sig==SIGFPE){
      printf("SIGFPE (event discarded)\n");
      flags[quit]=0;
      flags[bad_event]=1;
   }
  
}

void ParseCommandLine(int narg,char *argv[])
{
   int i;
   
   for(i=1;i<narg;i++){
      if(!strcmp(argv[i],"-d"))debug++;
      if(!strcmp(argv[i],"-v"))flags[verbose]=1;
      if(!strcmp(argv[i],"-b"))flags[batch_mode]=1;
      if(!strcmp(argv[i],"-g"))flags[graphics]=1;
      if(!strcmp(argv[i],"-gui"))flags[use_gui]=flags[graphics]=1;
      if(!strcmp(argv[i],"-p"))flags[pause_at_end]=1;
      if(!strcmp(argv[i],"-dt"))flags[graphics]=flags[draw_tracks]=1;
      if(!strcmp(argv[i],"-nct"))flags[dont_draw_charged_tracks]=1;
      if(!strcmp(argv[i],"-fh"))flags[fill_histograms]=1;
      if(!strcmp(argv[i],"-gs"))flags[global_section]=flags[fill_histograms]=1;
      if(!strcmp(argv[i],"-ss"))flags[skip_simulation]=1;
      if(!strcmp(argv[i],"-go"))flags[generate_only]=1;
      if(!strcmp(argv[i],"-sac"))flags[stop_at_calorimeter]=1;
      if(!strcmp(argv[i],"-gz"))flags[output_gzip]=1;
      if(!strcmp(argv[i],"-sv")){
         if(i>=narg-1){printf("\n\t\"-sv\" requires an argument.\n");exit(-1);}
         sscanf(argv[i+1],"%f %f %f %f %f %f %f"
            ,&vtheta,&vphi,&vpsi,&vu,&vv,&vsu,&vsv);
         flags[graphics]=1;
         i++;
      }
      if(!strncmp(argv[i],"-T",2))run_time=atof(&argv[i][2]);
      if(!strncmp(argv[i],"-G",2))GEOMETRY_TYPE=atoi(&argv[i][2]);
      if(!strncmp(argv[i],"-E",2))EVENT_WRITE_TRHESHOLD=atof(&argv[i][2]);
      if(!strncmp(argv[i],"-gen",4))SelectGenerator(atoi(&argv[i][4]));


      if(!strcmp(argv[i],"-h"))PrintUsage();
      if(!strcmp(argv[i],"-help"))PrintUsage();
      if(!strcmp(argv[i],"--help"))PrintUsage();
   
   }
}
void OpenOutputFile(void)
{
   char *host_computer;
   FILE *file;
   int i;
   time_t t;
   struct tm *tm;
   char *tm_str,month_str[5];
   
   
   t=time(NULL);
   tm=localtime(&t);
   tm_str=ctime(&t);
   strncpy(month_str,&tm_str[4],3);month_str[3]=0;
   
   /* find a unique file name so we don't overwrite anything */
   for(i=1;i<1000;i++){
      host_computer=getenv("HOST");
      sprintf(OUTFILE,"psim_%s_%d_%d.D%2.2d.%s"
         ,month_str
         ,tm->tm_mday
         ,tm->tm_year>50 ? 1900+tm->tm_year:2000+tm->tm_year
         ,i
         ,flags[output_gzip] ? "gz":"hbook");
      file=fopen(OUTFILE,"r");
      if(file==NULL)break;
      fclose(file);
   }
   
   if(strlen(OUTFILE)==0)sprintf(OUTFILE,"simout%3.3d.hbook",gcflag_.idrun);
   printf(" saving to file: %s\n",OUTFILE);
   fflush(stdout);
   
   /* open output file (either hbook or gzip binary)*/
   if(flags[output_gzip]){
      gzfile=gzopen(OUTFILE,"wb");
      if(!gzfile){
         printf("\nError opening \"%s\" for output!\n\n",OUTFILE);
         return;      
      }
   }else{
      if(hropen(1,"LUN1",OUTFILE,"N",1024,0)){
         printf("\nError opening \"%s\" for output!\n\n",OUTFILE);
         return;
      }
      hcdir_("//LUN1"," ",strlen("//LUN1"),strlen(" "));
   }

   printf("Energy needed in calorimeter to write event: %f GeV\n"
   	,EVENT_WRITE_TRHESHOLD);
}


