#!/usr/bin/wish -f



proc gui_init {} {

   set win .buttons
   frame $win -relief groove -borderwidth 3
   pack $win -side bottom -fill x
   button $win.quit -text "Quit" -command {Stop ; destroy .}
   pack $win.quit -side right

   set win .top
   frame $win
   pack $win -side top -fill both
   
   set win .middle 
   frame $win -relief groove -borderwidth 3
   pack $win -side top -fill both

   # View parameters
   set win .top.left
   frame $win -relief groove -borderwidth 3
   pack $win -side left -fill both
   button $win.redraw -text "Redraw" -command {Redraw}
   pack $win.redraw -side bottom
   define_variable vtheta $win
   define_variable vphi $win
   define_variable vpsi $win
   define_variable vu $win
   define_variable vv $win
   define_variable vsu $win
   define_variable vsv $win

   # View flags
   set win .top.vflags
   frame $win -relief groove -borderwidth 3
   pack $win -side left -fill both
   checkbutton $win.hide -text "Hidden lines" -variable hide -anchor w -width 14
   checkbutton $win.shade -text "Shading" -variable shade -anchor w -width 14
   checkbutton $win.nct -text "Don't Draw\nCharged Tracks" \
      -variable nct -anchor w -width 14 -justify left
   checkbutton $win.nnt -text "Don't Draw\nNeutral Tracks" \
      -variable nnt -anchor w -width 14 -justify left
   checkbutton $win.rbe -text "Redraw between\nevents" \
      -variable rbe -anchor w -width 14 -justify left
   pack $win.hide $win.shade $win.nct $win.nnt $win.rbe -side top

   # predfined view buttons
   set win .buttons
   button $win.view1 -text "View1" -command {ResetView 1}
   button $win.view2 -text "View2" -command {ResetView 2}
   button $win.view3 -text "View3" -command {ResetView 3}
   button $win.view4 -text "View4" -command {ResetView 4}
   button $win.view5 -text "View5" -command {ResetView 5}
   button $win.view6 -text "View6" -command {ResetView 6}
   pack $win.view1 $win.view2 $win.view3 $win.view4 $win.view5 $win.view6 \
   	 -side left

   # Position parameters
   set win .top.pos
   frame $win -relief groove -borderwidth 3
   pack $win -side left -fill both
   button $win.redefine -text "Redefine Geometry" -command {RedefineGeometry}
   pack $win.redefine -side bottom
   define_variable target_z $win
   define_variable dipole_z $win
   define_variable calorimeter_z $win

   # Run Controls
   set win .middle.right
   frame $win -relief groove -borderwidth 3
   pack $win -side right -fill both
   
   frame $win.event
   pack $win.event -side top -fill x
   label $win.event.lab -text "Event:" -width 12
   label $win.event.num -textvariable ievent -width 20
   pack $win.event.lab $win.event.num -side left
   
   frame $win.eventtype
   pack $win.eventtype -side top -fill x
   label $win.eventtype.lab -text "Type:" -width 12
   label $win.eventtype.num -textvariable generator_name -width 20
   pack $win.eventtype.lab $win.eventtype.num -side left

   frame $win.eventsubtype
   pack $win.eventsubtype -side top -fill x
   label $win.eventsubtype.lab -text "Subtype:" -width 12
   label $win.eventsubtype.num -textvariable subtype_name -width 20
   pack $win.eventsubtype.lab $win.eventsubtype.num -side left

   frame $win.decaymech
   pack $win.decaymech -side top -fill x
   label $win.decaymech.lab -text "Decay mech:" -width 12
   label $win.decaymech.num -textvariable decaymech_name -width 20
   pack $win.decaymech.lab $win.decaymech.num -side left


   frame $win.npart
   pack $win.npart -side top -fill x
   label $win.npart.lab -text "Nparts:" -width 12
   label $win.npart.num -textvariable npart_infos  -width 20
   pack $win.npart.lab $win.npart.num -side left

   button $win.oneevent -text "One Event" -width 12 -command {OneEvent}
   button $win.run -text "Run" -width 12  -command {Run}
   button $win.stop -text "Stop" -width 12  -command {Stop}
   pack $win.stop $win.run $win.oneevent -side bottom -fill x


   # Generators
   set win .middle.generators
   frame $win -relief groove -borderwidth 3
   pack $win -side left
   label $win.lab -text "Generators" -width 20
   pack $win.lab -side top
   define_generator $win photon "Photon"
   define_generator $win electron "Electron"
   define_generator $win pair "Pair Production"
   define_generator $win pi0 "Primakoff pi0"
   define_generator $win eta_he4 "Eta on He4"
   define_generator $win eta_c12 "Eta on C12"
   define_generator $win eta_prime_c12 "Eta' on C12"
   define_generator $win eta_prime_he4 "Eta' on He4"
   define_generator $win rho "Rho (phasespace)"
   define_generator $win omega "Omega (phasespace)"

   # other controls
   set win .middle.othercontrols
   frame $win -relief groove -borderwidth 3
   pack $win -side left -fill both
   label $win.lab -text "Other Controls" -width 20
   pack $win.lab -side top
   button $win.resethistos -text "Reset Histos" -command {ResetHistograms}
   pack $win.resethistos -side bottom
   checkbutton $win.gen_only -text "Generate Only" -variable generate_only -anchor w
   pack $win.gen_only -side top
   checkbutton $win.stop_at_cal -text "Stop at Calorimeter" \
   	-variable stop_at_calorimeter  -anchor w
   pack $win.stop_at_cal -side top
}

proc define_variable {name win} {

   frame $win.$name
   pack $win.$name -side top
   label $win.$name.lab -text $name -width 14
   entry $win.$name.en -width 14 -textvariable $name
   pack $win.$name.lab $win.$name.en -side left
   
}

proc define_generator {win var lab} {
   checkbutton $win.$var -text $lab -variable gen_$var -width 20\
      -anchor w -command {ResetGenerators}
   pack $win.$var -side top
}


gui_init
wm title . "Psim Controls"

after 100 IdleThread
