

#include "psim.h"



int MLU1[256],MLU2[256];


void DefineGroups(void);
void ProgramMLUs(void);

/* minimum photon energy trigger is to accept */
/* (trigger should fire when both photons are */
/*  above this threshold).                    */
float trig_thresh=1.0;


/* Global variables indicating which discriminators fired */
/* with the last call to "trigger()".                     */
int trig1_discriminators_x;
int trig1_discriminators_y;


/**********************************************/
/* Initialize stuff for the trigger simulator */
/**********************************************/
void init_trigger(void)
{
   /* Linear Fan-in Groups */
   DefineGroups();
   
   /* Program MLUs */
   ProgramMLUs();


}

/*****************************************************/
/* Returns a 1 or 0 depending on whether the trigger */
/* "should" fire on this event. This test doesn't    */
/* neccessarily mean it wasn't a background event.   */
/*****************************************************/
int good_event(void *re)
{
   int good;
   part_info_t *pinfo=((raw_event_t*)re)->part_info;
   float dist,dX,dY;
   float unit=2.125; /* dimension of PbW04 block */
   float max,min;
   
   if(((raw_event_t*)re)->Npart_infos < 2)return 0;
   
   dX=pinfo[0].X-pinfo[1].X;
   dY=pinfo[0].Y-pinfo[1].Y;
   dist=sqrt((dX*dX) + (dY*dY));
   
   max=25.0*unit; /* outer fiducial edge */
   min=2.0*unit;  /* inner fiducial edge */
   good=	   ( dist>20.0)
      	&&	( fabs(pinfo[0].X)<max )
         && ( fabs(pinfo[0].Y)<max )
         && ( fabs(pinfo[1].X)<max )
         && ( fabs(pinfo[1].Y)<max )
         && ( fabs(pinfo[0].X)>min || fabs(pinfo[0].Y)>min)
         && ( fabs(pinfo[1].X)>min || fabs(pinfo[1].Y)>min)
         && ( pinfo[0].E>trig_thresh )
         && ( pinfo[1].E>trig_thresh )
         ;

   return good;
}

/************************************************************/
/* Define Group subsets. A "group" here represents a group  */
/* of detectors added together in a UVa120 linear fan-in.   */
/* Some groups will be added together in UVa125 adder/      */
/* discriminator modules. This is done in trigger() below.  */
/************************************************************/
void DefineGroups(void)
{
   subset *s;
   int i,M,k,*x,*y,*dx,*dy;
   char str[256];
   int first,last;


   /* outer */
   int  x1[]={  1, 10, 19,  1, 10, 19, 0};
   int  y1[]={  1,  1,  1,  5,  5,  5, 0};
   int dx1[]={  9,  9,  9,  9,  9,  9, 0};
   int dy1[]={  4,  4,  4,  4,  4,  4, 0};

   int  x2[]={  1, 20,  1, 20,  1, 20, 0};
   int  y2[]={  9,  9, 13, 13, 16, 16, 0};
   int dx2[]={  8,  8,  8,  8,  8,  8, 0};
   int dy2[]={  4,  4,  3,  3,  4,  4, 0};

   int  x3[]={  1, 10, 19,  1, 10, 19, 0};
   int  y3[]={ 20, 20, 20, 24, 24, 24, 0};
   int dx3[]={  9,  9,  9,  9,  9,  9, 0};
   int dy3[]={  4,  4,  4,  4,  4,  4, 0};
   
   /* inner */
   int  x4[]={  1,  6, 10, 14, 19, 0};
   int  y4[]={  1,  1,  1,  1,  1, 0};
   int dx4[]={  4,  4,  4,  4,  4, 0};
   int dy4[]={  8,  8,  8,  8,  8, 0};
   
   int  x5[]={  1,  7, 12, 17, 0};
   int  y5[]={  9,  9,  9,  9, 0};
   int dx5[]={  6,  5,  5,  6, 0};
   int dy5[]={  6,  6,  6,  6, 0};
   
   int  x6[]={  1,  6, 10, 14, 19, 0};
   int  y6[]={ 15, 15, 15, 15, 15, 0};
   int dx6[]={  4,  4,  4,  4,  4, 0};
   int dy6[]={  8,  8,  8,  8,  8, 0};


   printf("Defining Groups for trigger 1\n");


   /*---------------------------------------*/
   /* strips to determine Vertical position */
   /*---------------------------------------*/   
   M=0; /* reset group counter */
   
   k=-1; x=x1; y=y1; dx=dx1; dy=dy1;
   while(x[++k]){
      sprintf(str,"V%d",++M);
      AddBlockToSubset(AddSubset(str),x[k],y[k],dx[k],dy[k],0);
   }

   k=-1; x=x2; y=y2; dx=dx2; dy=dy2; 
   while(x[++k]){
      sprintf(str,"V%d",++M);
      AddBlockToSubset(AddSubset(str),x[k],y[k],dx[k],dy[k],0);
   }

   k=-1; x=x3; y=y3; dx=dx3; dy=dy3;    
   while(x[++k]){
      sprintf(str,"V%d",++M);
      AddBlockToSubset(AddSubset(str),x[k],y[k],dx[k],dy[k],0);
   }

   k=-1; x=x4; y=y4; dx=dx4; dy=dy4;
   while(x[++k]){
      sprintf(str,"V%d",++M);
      AddBlockToSubset(AddSubset(str),x[k],y[k],dx[k],dy[k],1);
   }
   
   k=-1; x=x5; y=y5; dx=dx5; dy=dy5;    
   while(x[++k]){
      sprintf(str,"V%d",++M);
      AddBlockToSubset(AddSubset(str),x[k],y[k],dx[k],dy[k],1);
   }

   k=-1; x=x6; y=y6; dx=dx6; dy=dy6; 
   while(x[++k]){
      sprintf(str,"V%d",++M);
      AddBlockToSubset(AddSubset(str),x[k],y[k],dx[k],dy[k],1);
   }
   
   /* add in irregular parts of certain inner groups */
   AddBlockToSubset(FindSubset("V19"), 5, 5,1,4,1);
   AddBlockToSubset(FindSubset("V20"), 5, 1,1,4,1);
   AddBlockToSubset(FindSubset("V22"),18, 1,1,4,1);
   AddBlockToSubset(FindSubset("V23"),18, 5,1,4,1);
   
   AddBlockToSubset(FindSubset("V28"), 5,15,1,4,1);
   AddBlockToSubset(FindSubset("V29"), 5,19,1,4,1);
   AddBlockToSubset(FindSubset("V31"),18,15,1,4,1);
   AddBlockToSubset(FindSubset("V32"),18,19,1,4,1);


   /*-----------------------------------------*/
   /* strips to determine Horizontal position */
   /*-----------------------------------------*/   
   M=0; /* reset group counter */
   
   k=-1; x=x1; y=y1; dx=dx1; dy=dy1;
   while(x[++k]){
      sprintf(str,"H%d",++M);
      AddBlockToSubset(AddSubset(str),y[k],x[k],dy[k],dx[k],0);
   }

   k=-1; x=x2; y=y2; dx=dx2; dy=dy2;
   while(x[++k]){
      sprintf(str,"H%d",++M);
      AddBlockToSubset(AddSubset(str),y[k],x[k],dy[k],dx[k],0);
   }

   k=-1; x=x3; y=y3; dx=dx3; dy=dy3;   
   while(x[++k]){
      sprintf(str,"H%d",++M);
      AddBlockToSubset(AddSubset(str),y[k],x[k],dy[k],dx[k],0);
   }

   k=-1; x=x4; y=y4; dx=dx4; dy=dy4;   
   while(x[++k]){
      sprintf(str,"H%d",++M);
      AddBlockToSubset(AddSubset(str),y[k],x[k],dy[k],dx[k],1);
   }

   k=-1; x=x5; y=y5; dx=dx5; dy=dy5;   
   while(x[++k]){
      sprintf(str,"H%d",++M);
      AddBlockToSubset(AddSubset(str),y[k],x[k],dy[k],dx[k],1);
   }

   k=-1; x=x6; y=y6; dx=dx6; dy=dy6;
   while(x[++k]){
      sprintf(str,"H%d",++M);
      AddBlockToSubset(AddSubset(str),y[k],x[k],dy[k],dx[k],1);
   }

   /* add in irregular parts of certain inner groups */
   AddBlockToSubset(FindSubset("H19"), 5, 5,4,1,1);
   AddBlockToSubset(FindSubset("H20"), 1, 5,4,1,1);
   AddBlockToSubset(FindSubset("H22"), 1,18,4,1,1);
   AddBlockToSubset(FindSubset("H23"), 5,18,4,1,1);
   
   AddBlockToSubset(FindSubset("H28"),15, 5,4,1,1);
   AddBlockToSubset(FindSubset("H29"),19, 5,4,1,1);
   AddBlockToSubset(FindSubset("H31"),15,18,4,1,1);
   AddBlockToSubset(FindSubset("H32"),19,18,4,1,1);


   if(debug>1){
      /* show subset lists */
      for(i=1;i<=729;i++){
         ListSubsets(i);
      }
      for(i=1001;i<=1484;i++){
         ListSubsets(i);
      }
   }
}


/****************************************************************/
/* Fill the arrays with the same values that would be programed */
/* in the memory look-up units used in the trigger electronics  */
/* The first stage MLUs will output three bits:                 */
/*                                                              */
/*  bit 1: two non-adjacent hits (definite trigger)             */
/*  bit 2: two adjacent hits     (possible trigger)             */
/*  bit 3: one hit               (possible trigger)             */
/****************************************************************/
void ProgramMLUs(void)
{
   int i,j,k;
   int tmp;
   
   /* initalize. 3rd bit is always set on MLU1 if any hit */
   for(i=0;i<256;i++)MLU2[i]=0;
   for(i=1;i<256;i++)MLU1[i]=1<<2;
   MLU1[0]=0;
   
   /* set bit 2 on MLU1 when two or more adjacent hits */
   for(i=0;i<7;i++){ /* cycle through all possible adjacent pairs */
      tmp=(1<<i) + (1<<(i+1)); /* set two bits */
      /* include all possible states of bits not in pair */
      for(j=0;j<256;j++)MLU1[tmp|j]|=1<<1;
   }
   
   /* set bit 1 whenever two bits set which are separated */
   /* by at least one bit */
   for(i=0;i<6;i++){
      for(j=i+2;j<8;j++){
         tmp=(1<<i) + (1<<j); /* two bits separated by one or more bits */
         for(k=0;k<256;k++)MLU1[tmp|k]|=1<<0;
      }
   }
   
   /* set bit 1 for all "definite triggers" */
   for(j=0;j<256;j++){
      if(j&(1<<0) || j&(1<<3))MLU2[j]|=1<<0;
   }
   
   /* set bit 1 for adjacent hits in both directions */
   for(j=0;j<256;j++){
      if(j&(1<<1) && j&(1<<4))MLU2[j]|=1<<0;
   }
   
}



int trigger(float Ethresh,int* XBITS,int* YBITS)
{
   float E,E1,E2,E3;
   int xbits,ybits;
   int nxbits,nybits;
   int i;
   char str[256];
   float Etrigger_thresh=Ethresh/2.0;
   int dx,dy;
   
   /* Note: 
   /* Ethresh is the minimum energy photon we wish to */
   /*  accept in our trigger.
   /*
   /* Etrigger_thresh is the discriminator threshold  */
   
   
   /*********************************************************/
   /* Emulate the trigger electronics.                      */
   /*********************************************************/
   xbits=ybits=0;
   nxbits=nybits=0;
   dx=dy=0;

   /* Outer Vertical Strips */
   E=FindSubset("V1")->E + FindSubset("V2")->E + FindSubset("V3")->E;
   if(E>Etrigger_thresh){ xbits|=(1<<0); dx|=1<<0 ; nxbits++; }

   E=FindSubset("V4")->E + FindSubset("V5")->E + FindSubset("V6")->E;
   if(E>Etrigger_thresh){ xbits|=(1<<1); dx|=1<<1 ; nxbits++; }

   E=FindSubset("V13")->E + FindSubset("V14")->E + FindSubset("V15")->E;
   if(E>Etrigger_thresh){ xbits|=(1<<5); dx|=1<<5 ; nxbits++; }

   E=FindSubset("V16")->E + FindSubset("V17")->E + FindSubset("V18")->E;
   if(E>Etrigger_thresh){ xbits|=(1<<6); dx|=1<<6 ; nxbits++; }

   /* Inner Vertical Strips */
   E1=	FindSubset("V19")->E + FindSubset("V20")->E + FindSubset("V21")->E
      + FindSubset("V22")->E + FindSubset("V23")->E;
   E2=	FindSubset("V7")->E + FindSubset("V8")->E;
   if(E1>Etrigger_thresh)dx|=1<<7;
   if(E2>Etrigger_thresh)dx|=1<<2;
   if(E1>Etrigger_thresh || E2>Etrigger_thresh)
      { xbits|=(1<<2); nxbits++; }

   E1=	FindSubset("V24")->E + FindSubset("V25")->E
      + FindSubset("V26")->E + FindSubset("V27")->E;
   E2=	FindSubset("V9")->E + FindSubset("V10")->E;
   if(E1>Etrigger_thresh)dx|=1<<8;
   if(E2>Etrigger_thresh)dx|=1<<3;
   if(E1>Etrigger_thresh || E2>Etrigger_thresh)
      { xbits|=(1<<3); nxbits++; }

   E1=	FindSubset("V28")->E + FindSubset("V29")->E + FindSubset("V30")->E
      + FindSubset("V31")->E + FindSubset("V32")->E;
   E2=	FindSubset("V11")->E + FindSubset("V12")->E;
   if(E1>Etrigger_thresh)dx|=1<<9;
   if(E2>Etrigger_thresh)dx|=1<<4;
   if(E1>Etrigger_thresh || E2>Etrigger_thresh)
      { xbits|=(1<<4); nxbits++; }

   /* Outer Horizontal Strips */
   E=FindSubset("H1")->E + FindSubset("H2")->E + FindSubset("H3")->E;
   if(E>Etrigger_thresh){ ybits|=(1<<0); dy|=1<<0 ; nybits++; }

   E=FindSubset("H4")->E + FindSubset("H5")->E + FindSubset("H6")->E;
   if(E>Etrigger_thresh){ ybits|=(1<<1); dy|=1<<1 ; nybits++; }

   E=FindSubset("H13")->E + FindSubset("H14")->E + FindSubset("H15")->E;
   if(E>Etrigger_thresh){ ybits|=(1<<5); dy|=1<<5 ; nybits++; }

   E=FindSubset("H16")->E + FindSubset("H17")->E + FindSubset("H18")->E;
   if(E>Etrigger_thresh){ ybits|=(1<<6); dy|=1<<6 ; nybits++; }

   /* Inner Horizontal Strips */
   E1=	FindSubset("H19")->E + FindSubset("H20")->E + FindSubset("H21")->E
      + FindSubset("H22")->E + FindSubset("H23")->E;
   E2=	FindSubset("H7")->E + FindSubset("H8")->E;
   if(E1>Etrigger_thresh)dy|=1<<7;
   if(E2>Etrigger_thresh)dy|=1<<2;
   if(E1>Etrigger_thresh || E2>Etrigger_thresh)
      { ybits|=(1<<2); nybits++; }

   E1=	FindSubset("H24")->E + FindSubset("H25")->E
      + FindSubset("H26")->E + FindSubset("H27")->E;
   E2=	FindSubset("H9")->E + FindSubset("H10")->E;
   if(E1>Etrigger_thresh)dy|=1<<8;
   if(E2>Etrigger_thresh)dy|=1<<3;
   if(E1>Etrigger_thresh || E2>Etrigger_thresh)
      { ybits|=(1<<3); nybits++; }

   E1=	FindSubset("H28")->E + FindSubset("H29")->E + FindSubset("H30")->E
      + FindSubset("H31")->E + FindSubset("H32")->E;
   E2=	FindSubset("H11")->E + FindSubset("H12")->E;
   if(E1>Etrigger_thresh)dy|=1<<9;
   if(E2>Etrigger_thresh)dy|=1<<4;
   if(E1>Etrigger_thresh || E2>Etrigger_thresh)
      { ybits|=(1<<4); nybits++; }
   

   /* Display bits if in debug mode */
   if(debug>1){
      printf("\n");
      printf(" xbits(%d): ",nxbits);
      for(i=0;i<8;i++)printf("%d ",(xbits>>i)&1);
      printf("\n ybits(%d): ",nybits);
      for(i=0;i<8;i++)printf("%d ",(ybits>>i)&1);
      printf("\n");
      if(debug>2){
         for(i=1;i<=32;i++){
            sprintf(str,"V%d",i); printf("\t%s:%3.2f  ",str,FindSubset(str)->E);
            sprintf(str,"H%d",i); printf("\t%s:%3.2f\n",str,FindSubset(str)->E);
         }
      }
   }   
 
   if(XBITS)*XBITS=xbits;
   if(YBITS)*YBITS=ybits;
   trig1_discriminators_x=dx;
   trig1_discriminators_y=dy;
   
   return MLU2[(MLU1[ybits&0xFF]<<3) + MLU1[xbits&0xFF]];;
}









