

#include "psim.h"



int MLU_T5[65536];


void init_trigger5(void);
void ProgramMLUs5(void);

int trig5_discriminators_xlo;
int trig5_discriminators_xhi;
int trig5_discriminators_ylo;
int trig5_discriminators_yhi;


/**********************************************/
/* Initialize stuff for the trigger simulator */
/**********************************************/
void init_trigger5(void)
{
   /* Linear Fan-in Groups */
   /* Trigger 5 uses the same groups as trigger 1 */
   /* DefineGroups(); */  
   
   /* Program MLUs */
   ProgramMLUs5();


}

void print_bits(int A,int num)
{
   int i;
   
   for(i=0;i<num;i++)printf("%s ",(A>>i)&1 ? "1":"0");

   printf("\n");
}



/****************************************************************/
/* Fill the arrays with the same values that would be programed
/* in the memory look-up units used in the trigger electronics 
/*
/* For this trigger, One 16 input MLU will be used for the 
/* X strips and one for the Y. The outputs will be OR'ed in a
/* NIM module.
/*
/* Bits 1-7  = high discriminator threshold
/* Bits 8-14 = low discriminator threshold
/*
/* The MLU will set it's output bit if one of the following
/* conditions is met:
/*
/* (2 non-adj. high)
/*
/* (2 adj. low) + (1 high)       where the 1 high isn't one 
/*                               of the lows
/* (2 adj. low) + (2 adj. low)   these can overlap by 1
/*
/*
/* Since these patterns must be accepted for events for which
/* any additional bits are set, the MLU is first programmed
/* with the exact patterns and then updated to also accept
/* all patterns with additional bits.
/****************************************************************/
void ProgramMLUs5(void)
{
   int i,j,k;
   int mlu[65536];
   
   printf("Programming MLU for trigger 5\n");
   

   /* initalize  */
   for(i=0;i<65536;i++)mlu[i]=MLU_T5[i]=0;
   
   /* (2 non-adj. high) */
   for(i=0;i<5;i++)
      for(j=i+2;j<7;j++)mlu[ (1<<i) +(1<<j)]=1;

   /* (2 adj. low) + (1 high) */
   for(i=0;i<6;i++)
      for(j=0;j<7;j++)
         if(j!=i && j!=(i+1))
            mlu[ (1<<(i+7)) +(1<<(i+7+1)) +(1<<j) ]=1;

   /* (2 adj. low) + (2 adj. low) */
   for(i=0;i<6;i++)
      for(j=0;j<6;j++)
         if(j!=i)
            mlu[ ( (1<<(i+7)) +(1<<(i+7+1)) ) | ( (1<<j) +(1<<(j+7+1)) )]=1;
   
   
   /* Accept all bit patterns matching the ones programed so far */
   for(j=0;j<(1<<14);j++)
      if(mlu[j])
         for(i=1;i<(1<<14);i++)MLU_T5[j|i]=1;
         
   
}



int trigger5(float Ethresh,int* XBITS,int* YBITS)
{
   float E,E1,E2,E3;
   int xbits,ybits;
   int xbits_lo,xbits_hi;
   int ybits_lo,ybits_hi;
   int i;
   char str[256];
   float Etrigger_thresh=Ethresh/2.0;
   
   /* Note: 
   /* Ethresh is the minimum energy photon we wish to */
   /*  accept in our trigger.
   /*
   /* Etrigger_thresh is the discriminator threshold  */
   
   
   /*********************************************************/
   /* Emulate the trigger electronics.                      */
   /* Use trigger 1 to determine which bits are set         */
   /* Trigger 1 uses a discriminator threshold of 1/2       */ 
   /* the threshold passed to it.                           */
   /* The high discriminator is set to 80% of Ethresh.      */
   /* The low threshold is set to 20% of Ethresh.           */
   /*********************************************************/

   /* High threshold = 800 meV */
   trigger(0.75*Ethresh*2.0,&xbits_hi,&ybits_hi);
   trig5_discriminators_xhi=trig1_discriminators_x;
   trig5_discriminators_yhi=trig1_discriminators_y;

   /* Low  threshold = 200 meV */
   trigger(0.25*Ethresh*2.0,&xbits_lo,&ybits_lo);
   trig5_discriminators_xlo=trig1_discriminators_x;
   trig5_discriminators_ylo=trig1_discriminators_y;
   
   xbits = xbits_hi + (xbits_lo<<7);
   ybits = ybits_hi + (ybits_lo<<7);

   if(XBITS)*XBITS=xbits;
   if(YBITS)*YBITS=ybits;

   return MLU_T5[xbits] || MLU_T5[ybits];
}







