

#include <time.h>
#include <stdio.h>
#include <stdlib.h>
#include <stdarg.h>
#include <math.h>
#include <ctype.h>
#include <unistd.h>
#include <signal.h>
#include <zlib.h>
#include <string.h>


#ifdef __cplusplus
#include <stdiostream.h>
#include <iostream.h>
#include <strstream.h>
#include <iomanip.h>
#endif /* __cplusplus */


#include <cern.h>
#include <kinematics.h>



#define NWPAWC  500000
#define NWGEAN 2000000

#ifndef PI
#define PI 3.14159
#endif /* PI */

#define real float

#define DEG2RAD 0.017453292

#define max_detectors 2048
#define max_sets      512

#ifdef hpux
#define random rand
#define srandom srand
#endif /*hpux*/

/* Some Global variables controlling the run */
#define BATCH  gcflax_.batch
#define NEVENT gcflag_.nevent
#define IEVENT gcflag_.ievent
#define IEORUN gcflag_.ieorun

#define ansi_escape		((char)0x1b)
#define ansi_black()		printf("%c%s",ansi_escape,"[30m")
#define ansi_red()		printf("%c%s",ansi_escape,"[31m")
#define ansi_green()		printf("%c%s",ansi_escape,"[32m")
#define ansi_blue()		printf("%c%s",ansi_escape,"[34m")
#define ansi_bold()		printf("%c%s",ansi_escape,"[1m")
#define ansi_normal()	printf("%c%s",ansi_escape,"[0m")
#define ansi_up(A)		printf("%c[%dA",ansi_escape,A)
#define ansi_down(A)		printf("%c[%dB",ansi_escape,A)
#define ansi_forward(A)	printf("%c[%dC",ansi_escape,A)
#define ansi_back(A)		printf("%c[%dD",ansi_escape,A)



/* minimum photon energy trigger is to accept */
extern float trig_thresh;
extern float EVENT_WRITE_TRHESHOLD;

/* generate random number between teo limits */
/* this routine lives in GENERATORS:generators.c */
float ranl(float lower,float upper);
float RANDOM(void);


/* Some function declarations */
void* write_binary_event(void);
void fill_histos(void* re);
int good_event(void *re); /* re is raw_event_t* which is declared below */
void draw_all(void);

void init_trigger(void);
void init_trigger2(void);
void init_trigger3(void);
void init_trigger4(void);
void init_trigger5(void);
int trigger(float Ethresh,int* XBITS,int* YBITS);
int trigger2(float Ethresh,int* XBITS,int* YBITS);
int trigger3(float Ethresh);
int trigger4(float Ethresh);
int trigger5(float Ethresh,int* XBITS,int* YBITS);


extern float vtheta,vphi,vpsi,vu,vv,vsu,vsv;
extern int HIDE,SHADE;
extern int debug;
extern vect4 initial_4vect,initial_position;
extern int BEAM_ENERGY;

extern int MLU1[256],MLU2[256];
extern int trig1_discriminators_x;
extern int trig1_discriminators_y;
extern int trig5_discriminators_xlo;
extern int trig5_discriminators_xhi;
extern int trig5_discriminators_ylo;
extern int trig5_discriminators_yhi;


extern char OUTFILE[256];
extern FILE *gzfile;

extern int GEOMETRY_TYPE;

/* General program flags */
enum flag_types{
   quit,
   pause_at_end,
   global_section,
   bad_event,
   skip_simulation,
   stop_at_calorimeter,
   generate_only,
   
   verbose,
   batch_mode,
   
   use_gui,
   graphics,
   hide_on,
   shade_on,
   draw_tracks,
   dont_draw_charged_tracks,
   dont_draw_neutral_tracks,
   
   output_gzip,
   
   fill_histograms,
   update_goods,

   pair_produce,

   NUMFLAGS
};
extern int flags[];

/* flags used during tracking */
enum tracking_flags{
   draw_track,

   pair_prod,
   pair_prod_in_tgt,
   
   calorimeter_hit,
   
   NUM_TRACKFLAGS
};
extern int track_flags[];


extern int nt1,nt2,nt3,nt4,nt5;
extern int event_type,event_subtype,event_decay_mech;
extern double run_time;
extern long N_EVENT,N_GOOD_EVENT;
extern float initial_photon_energy;
extern int num_generators;
extern unsigned long generator_range_start[256];
extern unsigned long generator_range_stop[256];
extern char generator_names[32][32];

extern float        photon_weight;
extern float      electron_weight;
extern float          pair_weight;
extern float           pi0_weight;
extern float       eta_He4_weight;
extern float       eta_C12_weight;
extern float eta_prime_C12_weight;
extern float eta_prime_He4_weight;
extern float           rho_weight;
extern float         omega_weight;

void SelectGenerator(int who);

extern float pair_e_X,pair_e_Y,pair_p_X,pair_p_Y;
extern float phot_1_X,phot_1_Y,phot_2_X,phot_2_Y;

extern int convert_endians;

extern int zero,one,two,three,four,five,six,seven,eight,nine,ten;
extern float fzero,fone,ftwo,fthree,ffour,ffive,fsix,fseven,feight,fnine,ften;

/* this data structure type can be used for the whole  */
/* detector and/or subsets of it. (e.g. the outer ring */
/* ,the insertion array, left half,...                 */
typedef struct
{
   int  HITS;              /* number of detectors in this subset hit        */
   real E;		   /* Total energy deposited in set                 */
   real T;		   /* Time set was hit                              */
   real Eelem[max_detectors];  /* total energy deposited in subset              */
   real Telem[max_detectors];  /* time (relative to start) when hit             */
   
   int  Num;	             /* Number of members in this subset */
   int  list[max_detectors]; /* list of subset members		 */
   char name[32];
}subset;

/* declare a global handle to an array of type "subsets"   */
/* and a number of elements (memory will be allocated below */
extern subset *sets[max_sets];
extern int nsets;
extern int announce_subsets;
subset* AddSubset(char *name);
subset* FindSubset(char *name);
int SubsetExists(char *name);
void AddBlockToSubset(subset *s,int x,int y,int dx,int dy,int inner);
void ListSubsets(int who);
int ProcessSubsets(float *SHWDET,float *TSHWDET);


/* struct to hold particle info. for particles hitting calorimeter */
typedef struct
{
   float X,Y,Z;
   float PX,PY,PZ;
   float E,p;
   float T; /* time of flight */
   int   type;
}part_info_t;
#define max_part_infos 20
extern int Npart_infos;
extern part_info_t part_info[max_part_infos];

/* data structure used in cluster finding */
typedef struct {
   int N;
   int who[256];
   float X,Y;
   float E;
}cluster_t;
#define max_clusters 16
extern cluster_t clusters[max_clusters];
extern int Nsaved_clusters;

/* data structure used for trigger rates ntuple */
typedef struct {
   float X1,Y1,Z1;
   float pid1,E1;
   float X2,Y2,Z2;
   float pid2,E2;
   float Ephoton,Edep;
   float dist;
   float good,trig,xbits,ybits;
   float trig2,trig3,trig4,trig5;
}trig_rate_t;
extern trig_rate_t trig_rate;

/* structure of raw binary data */
typedef struct {
   char header[16];
   long event;
   
   float SHWDET[2048];
   float TSHWDET[2048];
   
   int type;
   float Ephoton;
   float Edeposited;
   
   vect4 initial_4vect;   /* vect4 defined in kinematics.h */
   vect4 initial_position;
   
   part_info_t part_info[max_part_infos]; /* particle info as it enters calorimeter */
   int Npart_infos;
   
   int subtype;      /* eg Primakoff, Nucl, or interference */
   int decay_mech;   /* eg decay channel */
   
   char trailer[16];
}raw_event_t;

typedef struct {
	int index;
	float E;
	float t;
}block_hit_info_t;

/* Make certain Common blocks visible from C */

typedef struct{
   float cutmag[10];
   int MGNSTP;
}magcut_t;
extern magcut_t magcut_;

typedef struct{
   float posit[5];
}posito_t;
extern posito_t posito_;

typedef struct{
   float zpln[5];
}zplane_t;
extern zplane_t zplane_;

typedef struct{
   float egfrac[2];
}phbmen_t;
extern phbmen_t phbmen_;

typedef struct{
   float egamak;
}brmst1_t;
extern brmst1_t brmst1_;

typedef struct{
   float shwdet[2048];
   float tshwdet[2048];
}cheren_t;
extern cheren_t cheren_;

typedef struct{
   float edprsw,edairg,edttsw,edtot;
   int ncptf1,ncpsf5,ncptot;
}nascom_t;
extern nascom_t nascom_;

typedef struct{
   float pi0d[4];
   float lgwidth;
   float lglength;
   float inwidth;
   float inlength;
   float wrap;
   float airgap;
   int   geomtype;
}newdet_t;
extern newdet_t newdet_;

typedef struct{
   float halflg[5];
}quadhl_t;
extern quadhl_t quadhl_;

typedef struct{
   float strent[5];
}mstrnt_t;
extern mstrnt_t mstrnt_;

typedef struct{
   float thrpos[3];
}thargt_t;
extern thargt_t thargt_;





