




#include "psim.h"


#define Ethresh 0.0

char fname[256];
FILE *gzfile;
int count_only=0;
int quit_program=0;
extern int convert_endians;

void ParseArgs(int narg,char *argv[]);
int psim_readfile_init(char *fname);
int read_event(raw_event_t *re);
void EHandler(int sig);

int main(int narg,char *argv[])
{
   raw_event_t re;
   int i,j;
   int Nevents;
   time_t last_time=0L;
   part_info_t *p;
   static int initialized=0;
   
   /*Try and exit gracefully should any errors occur*/
   signal(SIGPIPE,EHandler);
   signal(SIGTERM,EHandler);
   signal(SIGINT,EHandler);
   signal(SIGFPE,EHandler);

   ParseArgs(narg,argv);

   /* open input file */
   psim_readfile_init(fname);
   
   /* read in events */
   while(read_event(&re)){
      Nevents++;
      if(quit_program)break;

		if(!initialized){
			initialized=1;
			printf("convert_endians=%d\n",convert_endians);
			if(convert_endians)
				printf("Converting endians\n");
			else
				printf("Not Converting endians\n");
		}

      
      if(count_only){
         if(time(NULL)!=last_time){
            printf("  %d\r",Nevents);fflush(stdout);
            last_time=time(NULL);
         }
         continue;
      }
      
      printf("\nEvent %d  event(generated):%ld  Ephoton:%4.3f  Edeposited:%4.3f\n"
         ,Nevents ,re.event ,re.Ephoton,re.Edeposited);
      printf("Initial 4-vect: E:%5.4g  px:%5.4g  py:%5.4g  pz:%5.4g\n"
         ,re.initial_4vect.E, re.initial_4vect.x, re.initial_4vect.y, re.initial_4vect.z);
      printf("Initial position: R:%5.4g  x:%5.4g  y:%5.4g  z:%5.4g\n"
         ,re.initial_position.E, re.initial_position.x, re.initial_position.y, re.initial_position.z);
		printf("type:%d  subtype:%d  decay_mech:%d\n"
			,re.type ,re.subtype ,re.decay_mech);

      if(re.SHWDET[2040]){
         printf("VAW(GeV):%4.3g   VAW_frac:%4.3f\n"
            ,(float)re.SHWDET[2040],(float)re.SHWDET[2040]/re.Ephoton);
         
      }
      printf("Npart_infos:%d\n",re.Npart_infos);
      for(i=0;i<re.Npart_infos;i++){
         p=&re.part_info[i];
         printf("\tpart_info[%d]: type:%s  \n",i,part_type_str(p->type));
         printf("\t   X:%5.4g  Y:%5.4g  Z:%5.4g  E:%5.4g  p:%5.4g  T:%5.4g\n"
            ,p->X,p->Y,p->Z,p->E,p->p,p->T*1.0E9);
         printf("\t   PX:%5.4g  PY:%5.4g  PZ:%5.4g\n",p->PX,p->PY,p->PZ);
      }

      for(i=0;i<2048;i++){
         if(re.SHWDET[i]>Ethresh){
            printf("\t%d:\tE(GeV): %4.3g\t T(ns): %3.1f\n"
               ,i
               ,re.SHWDET[i]
               ,re.TSHWDET[i]*1.0E9);
         
         }
      }
      
   }
   
   /* just in case we "counted only" so that events generated can be used */
   if(re.Npart_infos>100 || re.Npart_infos==0)ConvertEndian(&re);

   printf("                             \n\n");
   printf(" %d events processed total\n",Nevents);
   printf(" %ld events generated total\n",re.event);
   printf("done.\n");
   
   return 0;
}


void ParseArgs(int narg,char *argv[])
{
   int i;
   
   if(narg<2){
      printf("You must provide a filename!\n");
      exit(-1);
   }
   
   for(i=1;i<narg-1;i++){
      if(!strcmp(argv[i],"-c")){count_only=1;convert_endians=0;};
   }
   
   strcpy(fname,argv[narg-1]);
}


void EHandler(int sig)
{
   quit_program=1;
   printf("\nReport: signal received of type ");
   if(sig==SIGPIPE)printf("SIGPIPE\n");
   if(sig==SIGTERM)printf("SIGTERM\n");
   if(sig==SIGINT)printf("SIGINT\n");
   if(sig==SIGFPE)printf("SIGFPE\n");
  
}



