

#include "psim_view.h"


/* global variable declarations */
Tcl_Interp* interp;
char TCL_PATH[256]=DEFAULT_TCL_PATH;
char inputfilename[256]="";
int good,trig[6];
raw_event_t re;
int show_trig_group=1;
int group_orientation=0;
int continuous=0;
int READ_EVENT=1;
extern int GEOMETRY_TYPE;

extern int announce_subsets;

/* variables needed for linking */
int debug=0;

/* function declarations */
void ParseCommandLineArgs(int narg,char *argv[]);
char *Subtype_string(int type,int subtype);
char *Decaymech_string(int type,int decay_mech);

int main(int narg,char* argv[])
{
   char cmd[512],str[256],fname[256],*ptr;
   int i;
   static int one=1,two=2,three=3,four=4,five=5;
   
   /*-------------- Print version info etc --------*/
   printf("\n----------- psim_view----------- \n");
   printf("Alpha Version: \ncompiled: ");
   ansi_red();    printf("%s %s \n\n"
      ,__DATE__,__TIME__);  ansi_normal();
   for(i=1;i<narg;i++)if(!strcmp(argv[i],"-v"))return 0;
   
   /*--------------- initialize -----------------*/
   
   /* trigger stuff */
   announce_subsets=0;
   init_trigger();
   init_trigger2();
   init_trigger3();
   init_trigger4();
   init_trigger5();

   
   /* Tcl/Tk */
   interp=Tcl_CreateInterp();
   if(!interp){printf("Unable to create Tcl/Tk interpreter!\n");exit(-1);}
   Tcl_Init(interp);
   Tk_Init(interp);
   
   
   /* define path to tcl code directory */
   if(ptr=getenv("PSIM_VIEW_TCL_PATH"))
      sprintf(TCL_PATH,"%s",ptr);
   else {
      printf("\n No \"PSIM_VIEW_TCL_PATH\" environment variable set. Defaulting to\n");
      printf(" \"%s\".\n\n",TCL_PATH);
   }
   
   /* Read Tcl source and Link C-tcl routines */
   sprintf(fname,"%s/psim_view.tcl",TCL_PATH);
   if(Tcl_EvalFile(interp,fname)!=TCL_OK)
      printf("Error in \"%s\":\n%s\n",fname,interp->result);

#define TRFLAGS TCL_GLOBAL_ONLY | TCL_TRACE_WRITES
   Tcl_CreateCommand(interp,"UpdateGroupEnergy",&TCL_UpdateGroupEnergy,0,NULL);
   Tcl_CreateCommand(interp,"DrawEvent",&TCL_DrawEvent,0,NULL);

   Tcl_TraceVar(interp,"show_trig_group",TRFLAGS,ShowTrigGroup,0);
   Tcl_LinkVar(interp,"show_trig_group",(char*)&show_trig_group,TCL_LINK_INT);
   Tcl_TraceVar(interp,"group_orientation",TRFLAGS,ShowTrigGroup,0);
   Tcl_LinkVar(interp,"group_orientation",(char*)&group_orientation,TCL_LINK_INT);
   Tcl_LinkVar(interp,"continuous",(char*)&continuous,TCL_LINK_INT);
   Tcl_LinkVar(interp,"geometry_type",(char*)&GEOMETRY_TYPE,TCL_LINK_INT);
   
   /*---------- Parse Command line args ---------*/
   ParseCommandLineArgs(narg,argv);
   psim_readfile_init(inputfilename);

   /*----------------- main loop -----------------*/
   Tcl_Eval(interp,"Initialize");
   printf("Initialize:%s\n",interp->result);
   Tcl_Eval(interp,"DrawGrid");
   printf("DrawGird:%s\n",interp->result);
   Tcl_Eval(interp,"wm title . \"PrimEx Event Viewer\"");
   if(GEOMETRY_TYPE==0)Tcl_Eval(interp,".c.s.tg.t1 invoke");
   sprintf(cmd,".c.s.ei.fil configure -text \"file:%s\"",inputfilename);
   Tcl_Eval(interp,cmd);
   sprintf(cmd,".c.s.ei.geomtype configure -text \"Geometry type: %d\"",GEOMETRY_TYPE);
   Tcl_Eval(interp,cmd);
   Tk_MainLoop();
   
   
   /*----------- completion and clean-up ---------*/
   
   return 0;
}


/* this is to satisfy potential linking problems using various f77s */
int MAIN__(int narg,char* argv[]){return main(narg,argv);}
int __main(int narg,char* argv[]){return main(narg,argv);}


int TCL_DrawEvent(	 ClientData clientData
   			,Tcl_Interp *interp
            ,int argc,char *argv[])
{
   static long Nevents=0L;
   int i;
   char cmd[512],str[256];
   int x,y,f;
   int insertion;
   char fill[32];
   char outline[32];
   char TAG[]="event";
   int recnum=0;
   int trig1_xbits,trig1_ybits;
   int trig2_xbits,trig2_ybits;
   int trig5_xbits,trig5_ybits;
   
   /* read in events                      */
   if(READ_EVENT){
      if(!read_event(&re)){
         printf("\nEnd of File.\n");
         return TCL_OK;
      }
   }
   
   /* Event info */
   sprintf(cmd,".c.s.ei.event configure -text \"event: %6d\"",re.event);
   Tcl_Eval(interp,cmd);
   sprintf(cmd,".c.s.ei.type configure -text \"MC type: %d\"",re.type);
   Tcl_Eval(interp,cmd);
   sprintf(cmd,".c.s.ei.subtype configure -text \"MC subtype: %s\""
   	,Subtype_string(re.type,re.subtype));
   Tcl_Eval(interp,cmd);
   sprintf(cmd,".c.s.ei.decaymech configure -text \"MC decaymech: %s\""
   	,Decaymech_string(re.type,re.decay_mech));
   Tcl_Eval(interp,cmd);
   
   /* pre-detected particle info */
   sprintf(cmd,".c.s.ip.ephoton configure -text \"Ephoton: %4.3f GeV\""
      ,re.Ephoton);
   Tcl_Eval(interp,cmd);
   
   Nevents++;
   ProcessSubsets(re.SHWDET,re.TSHWDET);
   good=good_event(&re);
   trig[1]=trigger(trig_thresh,&trig1_xbits,&trig1_ybits);
   trig[2]=trigger2(trig_thresh,&trig2_xbits,&trig2_ybits);
   trig[3]=trigger3(trig_thresh);
   trig[4]=trigger4(trig_thresh);
   trig[5]=trigger5(trig_thresh,&trig5_xbits,&trig5_ybits);
   
   /* clear the former event */
   sprintf(cmd,".c.c delete %s",TAG);
   Tcl_Eval(interp,cmd);

   for(i=0;i<2047;i++){
      if(re.SHWDET[i+1]>0.0){
      
         /* determine position and color based on energy */
         if(GEOMETRY_TYPE==0){
         	if(i<1000){
            	x=i%27;
            	y=i/27;
            	insertion=0;
         	}else{
            	x=(i-1000)%22;
            	y=(i-1000)/22;
            	insertion=1;
         	}
         }else if(GEOMETRY_TYPE==1 || GEOMETRY_TYPE==2){
				x=i%35;
				y=i/35;
				insertion=0;
         }
         
         f=re.SHWDET[i+1]*31.0/2.0E-2;
         if(f>0xFF)f=0xFF;
         f=0xFF-f;
         if(f>0xD0)f=0xD0;
         sprintf(fill,"#%2.2x%2.2x%2.2x",f,0x00,f);
         /*printf("Color=\"%s\"  E=%f\n",fill,re.SHWDET[i]);*/
         sprintf(outline,"#%x%x%x",0,0,0);
         
         /* color the cell */
         sprintf(cmd,"DrawCell %d %d %s %s {%s rec%d} %d %d"
               ,x ,y ,fill, outline ,TAG, ++recnum,1,insertion);
         Tcl_Eval(interp,cmd);
         
         /* bind a command to display values with mouseovers */
         sprintf(cmd,".c.c bind rec%d <Enter> ",recnum);
         sprintf(cmd,"%s{.i.energy configure -text \"E: %4.2f MeV\""
            ,cmd,re.SHWDET[i+1]*1.0E3);
         sprintf(cmd,"%s ; .i.time configure -text \"T: %4.2f ns\""
            ,cmd,re.TSHWDET[i+1]*1.0E9);
         sprintf(cmd,"%s ; .i.id configure -text \"id: %3d\"}"
            ,cmd,i+1);
         Tcl_Eval(interp,cmd);
         
      }
   }
   
   sprintf(cmd,".c.s.di.edep configure -text \"Edeposited: %4.3f GeV\"",re.Edeposited);
   Tcl_Eval(interp,cmd);

   sprintf(cmd,".c.s.di.e1 configure -text \"E1: %4.3f GeV\"",re.part_info[0].E);
   Tcl_Eval(interp,cmd);

   sprintf(cmd,".c.s.di.e2 configure -text \"E2: %4.3f GeV\"",re.Npart_infos>1 ? re.part_info[1].E:0.0);
   Tcl_Eval(interp,cmd);

   sprintf(cmd,".c.s.di.e3 configure -text \"E3: %4.3f GeV\"",re.Npart_infos>2 ? re.part_info[2].E:0.0);
   Tcl_Eval(interp,cmd);

   sprintf(cmd,".c.s.di.e4 configure -text \"E4: %4.3f GeV\"",re.Npart_infos>3 ? re.part_info[3].E:0.0);
   Tcl_Eval(interp,cmd);

   sprintf(cmd,".c.s.di.npart configure -text \"Npart: %d\"",re.Npart_infos);
   Tcl_Eval(interp,cmd);
   
   sprintf(cmd,".c.s.di.efrac configure -text \"Fraction detected: %4.1f%%\"",100.0*re.Edeposited/re.Ephoton);
   Tcl_Eval(interp,cmd);
   
   sprintf(cmd,".c.s.di.eleak configure -text \"Energy leaked through: %4.2f MeV\"",re.SHWDET[2040]*1000.0);
   Tcl_Eval(interp,cmd);

   sprintf(cmd,".c.s.di.eleakfrac configure -text \"Fraction leaked through: %4.1f%%\""
      ,100.0*re.SHWDET[2040]/re.Ephoton);
   Tcl_Eval(interp,cmd);
   
   /* update the trig info labels */
   sprintf(cmd,".c.s.tg.good configure -text \"good: %d\"",good);
   Tcl_Eval(interp,cmd);
   for(i=1;i<=5;i++){
      sprintf(cmd,".c.s.tg.t%d configure -text \"trig%d: %d\"",i,i,trig[i]);
      Tcl_Eval(interp,cmd);
   }
   
   if(show_trig_group==1){
      /* trig1 x bits */
      str[0]=0;x=trig1_xbits;for(i=0;i<7;i++){strcat(str,x&0x1 ? "1 ":"0 ");x/=2;}
      sprintf(cmd,".c.s.tg.tb1x configure -text \"trig1 x   : %s\"",str);
      Tcl_Eval(interp,cmd);

      /* trig1 y bits */
      str[0]=0;x=trig1_ybits;for(i=0;i<7;i++){strcat(str,x&0x1 ? "1 ":"0 ");x/=2;}
      sprintf(cmd,".c.s.tg.tb1y configure -text \"trig1 y   : %s\"",str);
      Tcl_Eval(interp,cmd);

   }else if(show_trig_group==5){
      /* trig5 x hi bits */
      str[0]=0;x=trig5_xbits;for(i=0;i<7;i++){strcat(str,x&0x1 ? "1 ":"0 ");x/=2;}
      sprintf(cmd,".c.s.tg.tb5xh configure -text \"trig5 x hi: %s\"",str);
      Tcl_Eval(interp,cmd);

      /* trig5 x lo bits */
      str[0]=0;x=trig5_xbits>>7;for(i=0;i<7;i++){strcat(str,x&0x1 ? "1 ":"0 ");x/=2;}
      sprintf(cmd,".c.s.tg.tb5xl configure -text \"trig5 x lo: %s\"",str);
      Tcl_Eval(interp,cmd);

      /* trig5 y hi bits */
      str[0]=0;x=trig5_ybits;for(i=0;i<7;i++){strcat(str,x&0x1 ? "1 ":"0 ");x/=2;}
      sprintf(cmd,".c.s.tg.tb5yh configure -text \"trig5 y hi: %s\"",str);
      Tcl_Eval(interp,cmd);

      /* trig5 y lo bits */
      str[0]=0;x=trig5_ybits>>7;for(i=0;i<7;i++){strcat(str,x&0x1 ? "1 ":"0 ");x/=2;}
      sprintf(cmd,".c.s.tg.tb5yl configure -text \"trig5 y lo: %s\"",str);
      Tcl_Eval(interp,cmd);
   }
   
   if(continuous)Tcl_Eval(interp,"update;update;after idle DrawEvent");

   return TCL_OK;
}


void ParseCommandLineArgs(int narg,char *argv[])
{
   int i;
   
   for(i=1;i<narg;i++){
      if(argv[i][0]!='-')strcpy(inputfilename,argv[i]);
      if(!strncmp(argv[i],"-T",2))trig_thresh=atof(&argv[i][2]);
   }

   if(strlen(inputfilename)==0){
      printf("\nYou must provide a filename!\n\n");
      exit(-1);
   }
   
   printf("\ntrig_thresh set to %f GeV\n\n",trig_thresh);

}


#define TOTAL_CROSS  0x01
#define PRIM_CROSS   0x02
#define NUCL_CROSS   0x04
#define INTER_CROSS   0x08
char *Subtype_string(int type,int subtype)
{
	static char str[32];
	
	switch(type){
		case 3:
		case 4:
		case 5:
		case 6:
		case 7:
			if(subtype==TOTAL_CROSS)return "Total cross";
			if(subtype==PRIM_CROSS )return "Primakoff";
			if(subtype==NUCL_CROSS )return "Nuclear Coh.";
			if(subtype==INTER_CROSS)return "Interference";
			break;
		case 8:
		case 9:
			return "phasespace";
			break;
		default:
			sprintf(str,"%d",subtype);
			return str;
	}
	return "Unknown";
}

char *Decaymech_string(int type,int decay_mech)
{
	static char str[32];
	
	switch(type){
		case 3:
			if(decay_mech==0)return "pi0 -> 2gamma";
		case 4:
		case 5:
			if(decay_mech==0 )return "eta -> 2gamma";
			if(decay_mech==1 )return "eta -> 3pi0";
			if(decay_mech==2 )return "eta -> pi+ pi- pi0";
			if(decay_mech==3 )return "eta -> pi+ pi- gamma";
			break;
		case 6:
		case 7:
			if(decay_mech==0 )return "eta' -> pi+ pi- eta";
			if(decay_mech==1 )return "eta' -> rho gamma";
			if(decay_mech==2 )return "eta' -> 2pi0 eta";
			if(decay_mech==3 )return "eta' -> omega gamma";
			if(decay_mech==4 )return "eta' -> gamma gamma";
			break;
		case 8:
			return "rho -> pi+ pi-";
			break;
		case 9:
			if(decay_mech==0 )return "omega -> pi+ pi- pi0";
			if(decay_mech==1 )return "omega -> pi0 gamma";
			if(decay_mech==2 )return "omega -> pi+ pi-";
			break;
		default:
			sprintf(str,"%d",decay_mech);
			return str;
	}
	return "Unknown";
}







